@@ -23,17 +23,9 @@ use TYPO3\CMS\Backend\Backend\ToolbarItems\SystemInformationToolbarItem;
  * An event to enrich the system information toolbar in the TYPO3 Backend top toolbar
  * with various information
  */
-final class SystemInformationToolbarCollectorEvent
+final readonly class SystemInformationToolbarCollectorEvent
 {
-    /**
-     * @var SystemInformationToolbarItem
-     */
-    private $toolbarItem;
-
-    public function __construct(SystemInformationToolbarItem $toolbarItem)
-    {
-        $this->toolbarItem = $toolbarItem;
-    }
+    public function __construct(private SystemInformationToolbarItem $toolbarItem) {}
 
     public function getToolbarItem(): SystemInformationToolbarItem
     {
