@@ -15,57 +15,42 @@
 
 namespace TYPO3\CMS\Backend\Command;
 
+use Symfony\Component\Console\Attribute\AsCommand;
 use Symfony\Component\Console\Command\Command;
 use Symfony\Component\Console\Input\InputInterface;
 use Symfony\Component\Console\Output\OutputInterface;
 use Symfony\Component\Console\Style\SymfonyStyle;
-use TYPO3\CMS\Core\Core\Environment;
+use TYPO3\CMS\Backend\Authentication\BackendLocker;
 
 /**
  * Core function for unlocking the TYPO3 Backend
  */
+#[AsCommand('backend:unlock', 'Unlock the TYPO3 Backend')]
 class UnlockBackendCommand extends Command
 {
-    /**
-     * Configure the command by defining the name, options and arguments
-     */
-    protected function configure()
+    public function __construct(protected readonly BackendLocker $lockService, ?string $name = null)
     {
-        $this->setDescription('Unlock the TYPO3 Backend');
+        parent::__construct($name);
     }
 
     /**
      * Executes the command for removing the lock file
-     *
-     * @param InputInterface $input
-     * @param OutputInterface $output
-     * @return int
      */
-    protected function execute(InputInterface $input, OutputInterface $output)
+    protected function execute(InputInterface $input, OutputInterface $output): int
     {
         $io = new SymfonyStyle($input, $output);
         $io->title($this->getDescription());
-        $lockFile = $this->getLockFileName();
-        if (@is_file($lockFile)) {
-            unlink($lockFile);
-            if (@is_file($lockFile)) {
+        $lockFile = $this->lockService->getAbsolutePathToLockFile();
+        if ($this->lockService->isLocked()) {
+            $this->lockService->unlock();
+            if ($this->lockService->isLocked()) {
                 $io->caution('Could not remove lock file "' . $lockFile . '"!');
-                return 1;
+                return Command::FAILURE;
             }
             $io->success('Removed lock file "' . $lockFile . '".');
         } else {
             $io->note('No lock file "' . $lockFile . '" was found.' . LF . 'Hence no lock can be removed.');
         }
-        return 0;
-    }
-
-    /**
-     * Location of the file name
-     *
-     * @return string
-     */
-    protected function getLockFileName()
-    {
-        return Environment::getLegacyConfigPath() . '/LOCK_BACKEND';
+        return Command::SUCCESS;
     }
 }
