@@ -17,21 +17,13 @@ declare(strict_types=1);
 
 namespace TYPO3\CMS\Backend\Form\Element;
 
-use TYPO3\CMS\Backend\Form\NodeFactory;
-use TYPO3\CMS\Core\Imaging\IconRegistry;
-use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\StringUtility;
 
 /**
- * Generation of TCEform elements of the type "check"
+ * Render elements of TCA type="check" with renderType="checkboxToggle".
  */
 class CheckboxToggleElement extends AbstractFormElement
 {
-    /**
-     * @var IconRegistry
-     */
-    private $iconRegistry;
-
     /**
      * Default field information enabled for this element.
      *
@@ -55,7 +47,7 @@ class CheckboxToggleElement extends AbstractFormElement
         'otherLanguageContent' => [
             'renderType' => 'otherLanguageContent',
             'after' => [
-                'localizationStateSelector'
+                'localizationStateSelector',
             ],
         ],
         'defaultLanguageDifferences' => [
@@ -66,16 +58,6 @@ class CheckboxToggleElement extends AbstractFormElement
         ],
     ];
 
-    /**
-     * @param NodeFactory $nodeFactory
-     * @param array $data
-     */
-    public function __construct(NodeFactory $nodeFactory, array $data)
-    {
-        parent::__construct($nodeFactory, $data);
-        $this->iconRegistry = GeneralUtility::makeInstance(IconRegistry::class);
-    }
-
     /**
      * This will render a checkbox or an array of checkboxes
      *
@@ -87,29 +69,30 @@ class CheckboxToggleElement extends AbstractFormElement
 
         $elementHtml = '';
         $disabled = false;
-        if ($this->data['parameterArray']['fieldConf']['config']['readOnly']) {
+        if ($this->data['parameterArray']['fieldConf']['config']['readOnly'] ?? false) {
             $disabled = true;
         }
         // Traversing the array of items
-        $items = $this->data['parameterArray']['fieldConf']['config']['items'];
+        $items = $this->data['parameterArray']['fieldConf']['config']['items'] ?? [];
 
         $numberOfItems = count($items);
         if ($numberOfItems === 0) {
-            $items[] = ['', ''];
+            $items[] = ['label' => ''];
             $numberOfItems = 1;
         }
-        $formElementValue = (int)$this->data['parameterArray']['itemFormElValue'];
-        $cols = (int)$this->data['parameterArray']['fieldConf']['config']['cols'];
+        // The values in the array may be numeric strings, but we need real ints.
+        $formElementValue = (int)($this->data['parameterArray']['itemFormElValue'] ?? 0);
+        $cols = (int)($this->data['parameterArray']['fieldConf']['config']['cols'] ?? 0);
         if ($cols > 1) {
             [$colClass, $colClear] = $this->calculateColumnMarkup($cols);
-            $elementHtml .= '<div class="checkbox-row row">';
+            $elementHtml .= '<div class="row">';
             $counter = 0;
             // $itemKey is important here, because items could have been removed via TSConfig
             foreach ($items as $itemKey => $itemDefinition) {
-                $label = $itemDefinition[0];
+                $label = $itemDefinition['label'];
                 $elementHtml .=
-                    '<div class="checkbox-column ' . $colClass . '">'
-                    . $this->renderSingleCheckboxElement($label, $itemKey, $formElementValue, $numberOfItems, $this->data['parameterArray'], $disabled) .
+                    '<div class="' . $colClass . '">'
+                        . $this->renderSingleCheckboxElement($label, $itemKey, $formElementValue, $numberOfItems, $this->data['parameterArray'], $disabled) .
                     '</div>';
                 ++$counter;
                 if ($counter < $numberOfItems && !empty($colClear)) {
@@ -123,8 +106,8 @@ class CheckboxToggleElement extends AbstractFormElement
             $elementHtml .= '</div>';
         } else {
             $counter = 0;
-            foreach ($items as $itemKey => $itemDefinition) {
-                $label = $itemDefinition[0];
+            foreach ($items as $itemDefinition) {
+                $label = $itemDefinition['label'];
                 $elementHtml .= $this->renderSingleCheckboxElement($label, $counter, $formElementValue, $numberOfItems, $this->data['parameterArray'], $disabled);
                 ++$counter;
             }
@@ -145,18 +128,18 @@ class CheckboxToggleElement extends AbstractFormElement
         $html[] = '<div class="formengine-field-item t3js-formengine-field-item">';
         $html[] = $fieldInformationHtml;
         $html[] =   '<div class="form-wizards-wrap">';
-        $html[] =       '<div class="form-wizards-element">';
+        $html[] =       '<div class="form-wizards-item-element">';
         $html[] =           $elementHtml;
         $html[] =       '</div>';
         if (!$disabled && !empty($fieldWizardHtml)) {
-            $html[] =   '<div class="form-wizards-items-bottom">';
+            $html[] =   '<div class="form-wizards-item-bottom">';
             $html[] =       $fieldWizardHtml;
             $html[] =   '</div>';
         }
         $html[] =   '</div>';
         $html[] = '</div>';
 
-        $resultArray['html'] = implode(LF, $html);
+        $resultArray['html'] = $this->wrapWithFieldsetAndLegend(implode(LF, $html));
         return $resultArray;
     }
 
@@ -167,35 +150,38 @@ class CheckboxToggleElement extends AbstractFormElement
      * @param int $itemCounter Number of this element in the list of all elements
      * @param int $formElementValue Value of this element
      * @param int $numberOfItems Full number of items
-     * @param array $additionalInformation Information with additional configuration options.
+     * @param array $additionalInformation Information with additional configuration options
      * @param bool $disabled TRUE if form element is disabled
      * @return string Single element HTML
      */
     protected function renderSingleCheckboxElement($label, $itemCounter, $formElementValue, $numberOfItems, $additionalInformation, $disabled): string
     {
+        $languageService = $this->getLanguageService();
+        $parentLabel = $additionalInformation['fieldConf']['label'] ?? '';
         $config = $additionalInformation['fieldConf']['config'];
-        $inline = !empty($config['cols']) && $config['cols'] === 'inline';
+        $inline = ($config['cols'] ?? '') === 'inline';
         $invert = isset($config['items'][0]['invertStateDisplay']) && $config['items'][0]['invertStateDisplay'] === true;
         $checkboxParameters = $this->checkBoxParams(
             $additionalInformation['itemFormElName'],
             $formElementValue,
             $itemCounter,
             $numberOfItems,
-            implode('', $additionalInformation['fieldChangeFunc'])
+            $additionalInformation['fieldChangeFunc'] ?? [],
+            $invert
         );
-        $uniqueId = StringUtility::getUniqueId('_');
-        $checkboxId = $additionalInformation['itemFormElID'] . '_' . $itemCounter . $uniqueId;
+        $checkboxId = htmlspecialchars(StringUtility::getUniqueId('formengine-check-toggle-') . '-' . $itemCounter);
         return '
-            <div class="checkbox checkbox-type-toggle' . ($invert ? ' checkbox-invert' : '') . ($inline ? ' checkbox-inline' : '') . (!$disabled ? '' : ' disabled') . '">
+            <div class="form-check form-switch' . ($inline ? ' form-check-inline' : '') . (!$disabled ? '' : ' disabled') . '">
                 <input type="checkbox"
-                    class="checkbox-input"
+                    class="form-check-input"
                     value="1"
+                    role="switch"
                     data-formengine-input-name="' . htmlspecialchars($additionalInformation['itemFormElName']) . '"
                     ' . $checkboxParameters . '
                     ' . (!$disabled ? '' : ' disabled="disabled"') . '
                     id="' . $checkboxId . '" />
-                <label class="checkbox-label" for="' . $checkboxId . '">
-                    <span class="checkbox-label-text">' . $this->appendValueToLabelInDebugMode(($label ? htmlspecialchars($label) : '&nbsp;'), $formElementValue) . '</span>
+                <label class="form-check-label" for="' . $checkboxId . '">
+                    ' . $this->appendValueToLabelInDebugMode(($label ? htmlspecialchars($label) : ''), $formElementValue) . '<span class="visually-hidden">' . htmlspecialchars($parentLabel) . '</span>' . '
                 </label>
             </div>';
     }
