@@ -16,7 +16,8 @@
 namespace TYPO3\CMS\Backend\Form\FormDataProvider;
 
 use TYPO3\CMS\Backend\Form\FormDataProviderInterface;
-use TYPO3\CMS\Core\Utility\GeneralUtility;
+use TYPO3\CMS\Backend\Form\Processor\SelectItemProcessor;
+use TYPO3\CMS\Core\Schema\Struct\SelectItem;
 use TYPO3\CMS\Core\Utility\MathUtility;
 
 /**
@@ -24,10 +25,13 @@ use TYPO3\CMS\Core\Utility\MathUtility;
  */
 class TcaSelectItems extends AbstractItemProvider implements FormDataProviderInterface
 {
+    public function __construct(
+        private readonly SelectItemProcessor $selectItemProcessor,
+    ) {}
+
     /**
      * Resolve select items
      *
-     * @param array $result
      * @return array
      * @throws \UnexpectedValueException
      */
@@ -52,7 +56,6 @@ class TcaSelectItems extends AbstractItemProvider implements FormDataProviderInt
                 $fieldConfig['config']['maxitems'] = 99999;
             }
 
-            $fieldConfig['config']['items'] = $this->addItemsFromSpecial($result, $fieldName, $fieldConfig['config']['items']);
             $fieldConfig['config']['items'] = $this->addItemsFromFolder($result, $fieldName, $fieldConfig['config']['items']);
 
             $fieldConfig['config']['items'] = $this->addItemsFromForeignTable($result, $fieldName, $fieldConfig['config']['items']);
@@ -91,7 +94,10 @@ class TcaSelectItems extends AbstractItemProvider implements FormDataProviderInt
             $result['databaseRow'][$fieldName] = $currentDatabaseValuesArray;
 
             // add item values as keys to determine which items are stored in the database and should be preselected
-            $itemArrayValues = array_column($fieldConfig['config']['items'], 1);
+            $itemArrayValues = array_column(
+                array_map(fn(SelectItem|array $item): array => $item instanceof SelectItem ? $item->toArray() : $item, $fieldConfig['config']['items']),
+                'value'
+            );
             $itemArray = array_fill_keys(
                 $itemArrayValues,
                 $fieldConfig['config']['items']
@@ -117,7 +123,7 @@ class TcaSelectItems extends AbstractItemProvider implements FormDataProviderInt
             // Keys may contain table names, so a numeric array is created
             $fieldConfig['config']['items'] = array_values($fieldConfig['config']['items']);
 
-            $fieldConfig['config']['items'] = $this->groupAndSortItems(
+            $fieldConfig['config']['items'] = $this->selectItemProcessor->groupAndSortItems(
                 $fieldConfig['config']['items'],
                 $fieldConfig['config']['itemGroups'] ?? [],
                 $fieldConfig['config']['sortItems'] ?? []
@@ -145,7 +151,7 @@ class TcaSelectItems extends AbstractItemProvider implements FormDataProviderInt
     {
         // Early return if there are no items or invalid values should not be displayed
         if (empty($fieldConf['config']['items'])
-            || $fieldConf['config']['renderType'] !== 'selectSingle'
+            || ($fieldConf['config']['renderType'] ?? '') !== 'selectSingle'
             || ($result['pageTsConfig']['TCEFORM.'][$table . '.'][$fieldName . '.']['disableNoMatchingValueElement'] ?? false)
             || ($fieldConf['config']['disableNoMatchingValueElement'] ?? false)
         ) {
@@ -155,20 +161,32 @@ class TcaSelectItems extends AbstractItemProvider implements FormDataProviderInt
         $languageService = $this->getLanguageService();
         $noMatchingLabel = isset($result['pageTsConfig']['TCEFORM.'][$table . '.'][$fieldName . '.']['noMatchingValue_label'])
             ? $languageService->sL(trim($result['pageTsConfig']['TCEFORM.'][$table . '.'][$fieldName . '.']['noMatchingValue_label']))
-            : '[ ' . $languageService->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:labels.noMatchingValue') . ' ]';
+            : '[ ' . $languageService->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:labels.noMatchingLabel') . ' ]';
 
         $unmatchedValues = array_diff(
             array_values($databaseValues),
-            array_column($fieldConf['config']['items'], 1),
-            array_column($removedItems, 1)
+            array_column(
+                array_map(
+                    fn(SelectItem|array $item): array => $item instanceof SelectItem ? $item->toArray() : $item,
+                    $fieldConf['config']['items']
+                ),
+                'value'
+            ),
+            array_column(
+                array_map(
+                    fn(SelectItem|array $item): array => $item instanceof SelectItem ? $item->toArray() : $item,
+                    $removedItems
+                ),
+                'value'
+            )
         );
 
         foreach ($unmatchedValues as $unmatchedValue) {
             $invalidItem = [
-                @sprintf($noMatchingLabel, $unmatchedValue),
-                $unmatchedValue,
-                null,
-                'none' // put it in the very first position in the "none" group
+                'label' => @sprintf($noMatchingLabel, $unmatchedValue),
+                'value' => $unmatchedValue,
+                'icon' => null,
+                'group' => 'none', // put it in the very first position in the "none" group
             ];
             array_unshift($fieldConf['config']['items'], $invalidItem);
         }
@@ -179,145 +197,10 @@ class TcaSelectItems extends AbstractItemProvider implements FormDataProviderInt
     /**
      * Determines whether the current field is a valid target for this DataProvider
      *
-     * @param array $fieldConfig
      * @return bool
      */
     protected function isTargetRenderType(array $fieldConfig)
     {
-        return $fieldConfig['config']['renderType'] !== 'selectTree';
-    }
-
-    /**
-     * Is used when --div-- elements in the item list are used, or if groups are defined via "groupItems" config array.
-     *
-     * This method takes the --div-- elements out of the list, and adds them to the group lists.
-     *
-     * A main "none" group is added, which is always on top, when items are not set to be in a group.
-     * All items without a groupId - which is defined by the fourth key of an item in the item array - are added
-     * to the "none" group, or to the last group used previously, to ensure ordering as much as possible as before.
-     *
-     * Then the found groups are iterated over the order in the [itemGroups] list,
-     * and items within a group can be sorted via "sortOrders" configuration.
-     *
-     * All grouped items are then "flattened" out and --div-- items are added for each group to keep backwards-compatibility.
-     *
-     * @param array $allItems all resolved items including the ones from foreign_table values. The group ID information can be found in fourth key [3] of an item.
-     * @param array $definedGroups [config][itemGroups]
-     * @param array $sortOrders [config][sortOrders]
-     * @return array
-     */
-    protected function groupAndSortItems(array $allItems, array $definedGroups, array $sortOrders): array
-    {
-        $groupedItems = [];
-        // Append defined groups at first, as their order is prioritized
-        $itemGroups = ['none' => ''];
-        foreach ($definedGroups as $groupId => $groupLabel) {
-            $itemGroups[$groupId] = $this->getLanguageService()->sL($groupLabel);
-        }
-        $currentGroup = 'none';
-        // Extract --div-- into itemGroups
-        foreach ($allItems as $key => $item) {
-            if ($item[1] === '--div--') {
-                // A divider is added as a group (existing groups will get their label overridden)
-                if (isset($item[3])) {
-                    $currentGroup = $item[3];
-                    $itemGroups[$currentGroup] = $item[0];
-                } else {
-                    $currentGroup = 'none';
-                }
-                continue;
-            }
-            // Put the given item in the currentGroup if no group has been given already
-            if (!isset($item[3])) {
-                $item[3] = $currentGroup;
-            }
-            $groupIdOfItem = !empty($item[3]) ? $item[3] : 'none';
-            // It is still possible to have items that have an "unassigned" group, so they are moved to the "none" group
-            if (!isset($itemGroups[$groupIdOfItem])) {
-                $itemGroups[$groupIdOfItem] = '';
-            }
-
-            // Put the item in its corresponding group (and create it if it does not exist yet)
-            if (!is_array($groupedItems[$groupIdOfItem] ?? null)) {
-                $groupedItems[$groupIdOfItem] = [];
-            }
-            $groupedItems[$groupIdOfItem][] = $item;
-        }
-        // Only "none" = no grouping used explicitly via "itemGroups" or via "--div--"
-        if (count($itemGroups) === 1) {
-            if (!empty($sortOrders)) {
-                $allItems = $this->sortItems($allItems, $sortOrders);
-            }
-            return $allItems;
-        }
-
-        // $groupedItems contains all items per group
-        // $itemGroups contains all groups in order of each group
-
-        // Let's add the --div-- items again ("unpacking")
-        // And use the group ordering given by the itemGroups
-        $finalItems = [];
-        foreach ($itemGroups as $groupId => $groupLabel) {
-            $itemsInGroup = $groupedItems[$groupId] ?? [];
-            if (empty($itemsInGroup)) {
-                continue;
-            }
-            // If sorting is defined, sort within each group now
-            if (!empty($sortOrders)) {
-                $itemsInGroup = $this->sortItems($itemsInGroup, $sortOrders);
-            }
-            // Add the --div-- if it is not the "none" default item
-            if ($groupId !== 'none') {
-                // Fall back to the groupId, if there is no label for it
-                $groupLabel = $groupLabel ?: $groupId;
-                $finalItems[] = [$groupLabel, '--div--', null, $groupId, null];
-            }
-            $finalItems = array_merge($finalItems, $itemsInGroup);
-        }
-        return $finalItems;
-    }
-
-    /**
-     * Sort given items by label or value or a custom user function built like
-     * "MyVendor\MyExtension\TcaSorter->sortItems" or a callable.
-     *
-     * @param array $items
-     * @param array $sortOrders should be something like like [label => desc]
-     * @return array the sorted items
-     */
-    protected function sortItems(array $items, array $sortOrders): array
-    {
-        foreach ($sortOrders as $order => $direction) {
-            switch ($order) {
-                case 'label':
-                    $direction = strtolower($direction);
-                    @usort(
-                        $items,
-                        function ($item1, $item2) use ($direction) {
-                            if ($direction === 'desc') {
-                                return strcasecmp($item1[0], $item2[0]) <= 0;
-                            }
-                            return strcasecmp($item1[0], $item2[0]);
-                        }
-                    );
-                    break;
-                case 'value':
-                    $direction = strtolower($direction);
-                    @usort(
-                        $items,
-                        function ($item1, $item2) use ($direction) {
-                            if ($direction === 'desc') {
-                                return strcasecmp($item1[1], $item2[1]) <= 0;
-                            }
-                            return strcasecmp($item1[1], $item2[1]);
-                        }
-                    );
-                    break;
-                default:
-                    $reference = null;
-                    GeneralUtility::callUserFunction($direction, $items, $reference);
-            }
-        }
-        return $items;
+        return ($fieldConfig['config']['renderType'] ?? '') !== 'selectTree';
     }
 }
