@@ -20,9 +20,6 @@ namespace TYPO3\CMS\Backend\Preview;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 
 /**
- * Class StandardPreviewRendererResolver
- *
- * Default implementation of PreviewRendererResolverInterface.
  * Scans TCA configuration to detect:
  *
  * - TCA.$table.types.$typeFromTypeField.previewRenderer
@@ -31,13 +28,12 @@ use TYPO3\CMS\Core\Utility\GeneralUtility;
  * Depending on which one is defined and checking the first, type-specific
  * variant first.
  */
-class StandardPreviewRendererResolver implements PreviewRendererResolverInterface
+class StandardPreviewRendererResolver
 {
     /**
      * @param string $table The name of the table the returned PreviewRenderer must work with
      * @param array $row A record from $table which will be previewed - allows returning a different PreviewRenderer based on record attributes
      * @param int $pageUid The UID of the page on which the preview will be rendered - allows returning a different PreviewRenderer based on for example pageTSconfig
-     * @return PreviewRendererInterface
      * @throws \UnexpectedValueException
      * @throws \RuntimeException
      */
@@ -50,6 +46,7 @@ class StandardPreviewRendererResolver implements PreviewRendererResolverInterfac
             $tcaTypeOfRow = $row[$tcaTypeField];
             $typeConfiguration = $tca['types'][$tcaTypeOfRow] ?? [];
 
+            // @deprecated Remove in v14, when "sub types" are removed altogether
             $subTypeValueField = $typeConfiguration['subtype_value_field'] ?? null;
             if (!empty($typeConfiguration['previewRenderer'])) {
                 if (!empty($subTypeValueField) && is_array($typeConfiguration['previewRenderer'])) {
@@ -68,14 +65,12 @@ class StandardPreviewRendererResolver implements PreviewRendererResolverInterfac
         }
 
         if (!$previewRendererClassName) {
-
             // Table either has no type field or no custom preview renderer was defined for the type.
             // Use table's standard renderer if any is defined.
             $previewRendererClassName = $tca['ctrl']['previewRenderer'] ?? null;
         }
 
-        if (!empty($previewRendererClassName)) {
-            /** @var string $previewRendererClassName */
+        if (is_string($previewRendererClassName) && $previewRendererClassName !== '') {
             if (!is_a($previewRendererClassName, PreviewRendererInterface::class, true)) {
                 throw new \UnexpectedValueException(
                     sprintf(
