@@ -1,5 +1,4 @@
 <?php
-namespace TYPO3\CMS\Backend\Routing\Exception;
 
 /*
  * This file is part of the TYPO3 CMS project.
@@ -14,9 +13,11 @@ namespace TYPO3\CMS\Backend\Routing\Exception;
  * The TYPO3 project - inspiring people to share!
  */
 
+namespace TYPO3\CMS\Backend\Routing\Exception;
+
+use TYPO3\CMS\Core\Exception;
+
 /**
  * Exception thrown when a resource was not found.
  */
-class ResourceNotFoundException extends \TYPO3\CMS\Core\Exception
-{
-}
+class ResourceNotFoundException extends Exception {}
