@@ -21,29 +21,34 @@ use TYPO3\CMS\Core\Utility\GeneralUtility;
 /**
  * Tree Node
  */
-class TreeNode implements ComparableNodeInterface, \Serializable
+class TreeNode implements ComparableNodeInterface
 {
     /**
      * Node Identifier
      *
-     * @var string
+     * @var string|int
      */
     protected $id = '';
 
     /**
      * Parent Node
      *
-     * @var \TYPO3\CMS\Backend\Tree\TreeNode
+     * @var TreeNode|null
      */
     protected $parentNode;
 
     /**
      * Child Nodes
      *
-     * @var \TYPO3\CMS\Backend\Tree\TreeNodeCollection
+     * @var TreeNodeCollection|null
      */
     protected $childNodes;
 
+    /**
+     * @internal This is part of the category tree performance hack.
+     */
+    protected array $additionalData = [];
+
     /**
      * Constructor
      *
@@ -54,15 +59,13 @@ class TreeNode implements ComparableNodeInterface, \Serializable
      */
     public function __construct(array $data = [])
     {
-        if (!empty($data)) {
+        if ($data !== []) {
             $this->dataFromArray($data);
         }
     }
 
     /**
      * Sets the child nodes collection
-     *
-     * @param \TYPO3\CMS\Backend\Tree\TreeNodeCollection $childNodes
      */
     public function setChildNodes(TreeNodeCollection $childNodes)
     {
@@ -83,7 +86,7 @@ class TreeNode implements ComparableNodeInterface, \Serializable
     /**
      * Returns child nodes collection
      *
-     * @return \TYPO3\CMS\Backend\Tree\TreeNodeCollection
+     * @return TreeNodeCollection
      */
     public function getChildNodes()
     {
@@ -106,7 +109,7 @@ class TreeNode implements ComparableNodeInterface, \Serializable
     /**
      * Sets the identifier
      *
-     * @param string $id
+     * @param string|int $id
      */
     public function setId($id)
     {
@@ -116,7 +119,7 @@ class TreeNode implements ComparableNodeInterface, \Serializable
     /**
      * Returns the identifier
      *
-     * @return string
+     * @return string|int
      */
     public function getId()
     {
@@ -126,9 +129,9 @@ class TreeNode implements ComparableNodeInterface, \Serializable
     /**
      * Sets the parent node
      *
-     * @param \TYPO3\CMS\Backend\Tree\TreeNode|null $parentNode
+     * @param TreeNode|null $parentNode
      */
-    public function setParentNode(\TYPO3\CMS\Backend\Tree\TreeNode $parentNode = null)
+    public function setParentNode(?TreeNode $parentNode = null)
     {
         $this->parentNode = $parentNode;
     }
@@ -136,7 +139,7 @@ class TreeNode implements ComparableNodeInterface, \Serializable
     /**
      * Returns the parent node
      *
-     * @return \TYPO3\CMS\Backend\Tree\TreeNode
+     * @return TreeNode
      */
     public function getParentNode()
     {
@@ -146,10 +149,9 @@ class TreeNode implements ComparableNodeInterface, \Serializable
     /**
      * Compares a node if it's identical to another node by the id property.
      *
-     * @param \TYPO3\CMS\Backend\Tree\TreeNode $other
      * @return bool
      */
-    public function equals(\TYPO3\CMS\Backend\Tree\TreeNode $other)
+    public function equals(TreeNode $other)
     {
         return $this->id == $other->getId();
     }
@@ -162,7 +164,7 @@ class TreeNode implements ComparableNodeInterface, \Serializable
      * -1 if its smaller than the other one
      * 0 if its equal
      *
-     * @param \TYPO3\CMS\Backend\Tree\TreeNode $other
+     * @param TreeNode $other
      * @return int See description above
      */
     public function compareTo($other)
@@ -173,6 +175,14 @@ class TreeNode implements ComparableNodeInterface, \Serializable
         return $this->id > $other->getId() ? 1 : -1;
     }
 
+    /**
+     * @internal This is part of the category tree performance hack
+     */
+    public function getAdditionalData(): array
+    {
+        return $this->additionalData;
+    }
+
     /**
      * Returns the node in an array representation that can be used for serialization
      *
@@ -183,7 +193,7 @@ class TreeNode implements ComparableNodeInterface, \Serializable
     {
         $arrayRepresentation = [
             'serializeClassName' => static::class,
-            'id' => $this->id
+            'id' => $this->id,
         ];
         if ($this->parentNode !== null) {
             $arrayRepresentation['parentNode'] = $this->parentNode->toArray(false);
@@ -205,7 +215,7 @@ class TreeNode implements ComparableNodeInterface, \Serializable
      */
     public function dataFromArray($data)
     {
-        $this->setId($data['id']);
+        $this->setId($data['id'] ?? $data['uid']);
         if (isset($data['parentNode']) && $data['parentNode'] !== '') {
             /** @var TreeNode $parentNode */
             $parentNode = GeneralUtility::makeInstance($data['parentNode']['serializeClassName'], $data['parentNode']);
@@ -216,27 +226,25 @@ class TreeNode implements ComparableNodeInterface, \Serializable
             $childNodes = GeneralUtility::makeInstance($data['childNodes']['serializeClassName'], $data['childNodes']);
             $this->setChildNodes($childNodes);
         }
+        // @todo: This is part of the category tree performance hack
+        $this->additionalData = $data;
     }
 
     /**
-     * Returns the serialized instance
-     *
-     * @return string
+     * Returns class state to be serialized.
      */
-    public function serialize()
+    public function __serialize(): array
     {
-        return serialize($this->toArray());
+        return $this->toArray();
     }
 
     /**
-     * Fills the current node with the given serialized informations
+     * Fills the current node with the given serialized information
      *
-     * @throws \TYPO3\CMS\Core\Exception if the deserialized object type is not identical to the current one
-     * @param string $serializedString
+     * @throws Exception if the deserialized object type is not identical to the current one
      */
-    public function unserialize($serializedString)
+    public function __unserialize(array $arrayRepresentation): void
     {
-        $arrayRepresentation = unserialize($serializedString);
         if ($arrayRepresentation['serializeClassName'] !== static::class) {
             throw new Exception('Deserialized object type is not identical!', 1294586646);
         }
