@@ -1,5 +1,7 @@
 <?php
 
+declare(strict_types=1);
+
 /*
  * This file is part of the TYPO3 CMS project.
  *
@@ -15,29 +17,19 @@
 
 namespace TYPO3\CMS\Backend\Tree\View;
 
+use Psr\Http\Message\ServerRequestInterface;
 use TYPO3\CMS\Backend\Routing\UriBuilder;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 
 /**
- * Position map class for moving pages,
+ * Position map class for moving pages.
+ *
  * @internal This class is a TYPO3 Backend implementation and is not considered part of the Public TYPO3 API.
  */
 class PageMovingPagePositionMap extends PagePositionMap
 {
     /**
-     * @var string
-     */
-    public $l_insertNewPageHere = 'movePageToHere';
-
-    /**
-     * Page tree implementation class name
-     *
-     * @var string
-     */
-    protected $pageTreeClassName = PageTreeView::class;
-
-    /**
-     * Creates the onclick event for the insert-icons.
+     * Creates the link target for the insert-icons.
      *
      * @param int $pid The pid.
      * @param int $newPagePID New page id.
@@ -59,9 +51,20 @@ class PageMovingPagePositionMap extends PagePositionMap
      * @param array $rec Page record (?)
      * @return string Wrapped title.
      */
-    public function linkPageTitle($str, $rec)
+    public function linkPageTitle($str, $rec, ServerRequestInterface $request)
     {
-        $url = GeneralUtility::linkThisScript(['uid' => (int)$rec['uid'], 'moveUid' => $this->moveUid]);
+        $uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);
+        $queryParams = $request->getQueryParams();
+        $url = (string)$uriBuilder->buildUriFromRoute(
+            'move_element',
+            [
+                'table' => $queryParams['table'] ?? '',
+                'uid' => (int)($rec['uid'] ?? 0),
+                'moveUid' => $this->moveUid,
+                'makeCopy' => $queryParams['makeCopy'] ?? 0,
+                'returnUrl' => $queryParams['returnUrl'] ?? '',
+            ]
+        );
         return '<a href="' . htmlspecialchars($url) . '">' . $str . '</a>';
     }
 
@@ -77,4 +80,14 @@ class PageMovingPagePositionMap extends PagePositionMap
     {
         return parent::boldTitle($t_code, $dat, (int)$this->moveUid);
     }
+
+    /**
+     * Get label, htmlspecialchars()'ed
+     *
+     * @return string The localized label for "move page here"
+     */
+    protected function insertlabel()
+    {
+        return htmlspecialchars($this->getLanguageService()->sL('LLL:EXT:core/Resources/Private/Language/locallang_misc.xlf:movePageToHere'));
+    }
 }
