@@ -9,6 +9,7 @@ return [
         'typeicon_column' => 'type',
         'typeicon_classes' => [
             'staticText' => 'mimetypes-text-html',
+            'asset' => 'mimetypes-text-html',
             'uri' => 'apps-pagetree-page-content-from-page',
         ],
     ],
@@ -17,7 +18,8 @@ return [
             'label' => 'LLL:EXT:backend/Resources/Private/Language/locallang_siteconfiguration_tca.xlf:site_route.route',
             'config' => [
                 'type' => 'input',
-                'eval' => 'required, trim',
+                'required' => true,
+                'eval' => 'trim',
                 'placeholder' => 'LLL:EXT:backend/Resources/Private/Language/locallang_siteconfiguration_tca.xlf:site_route.route.placeholder',
                 'valuePicker' => [
                     'items' => [
@@ -33,9 +35,10 @@ return [
                 'renderType' => 'selectSingle',
                 'required' => true,
                 'items' => [
-                    ['', ''],
-                    ['LLL:EXT:backend/Resources/Private/Language/locallang_siteconfiguration_tca.xlf:site_route.staticText', 'staticText'],
-                    ['LLL:EXT:backend/Resources/Private/Language/locallang_siteconfiguration_tca.xlf:site_route.source', 'uri']
+                    ['label' => '', 'value' => ''],
+                    ['label' => 'LLL:EXT:backend/Resources/Private/Language/locallang_siteconfiguration_tca.xlf:site_route.staticText', 'value' => 'staticText'],
+                    ['label' => 'LLL:EXT:backend/Resources/Private/Language/locallang_siteconfiguration_tca.xlf:site_route.source', 'value' => 'uri'],
+                    ['label' => 'LLL:EXT:backend/Resources/Private/Language/locallang_siteconfiguration_tca.xlf:site_route.asset', 'value' => 'asset'],
                 ],
             ],
         ],
@@ -43,13 +46,15 @@ return [
             'label' => 'LLL:EXT:backend/Resources/Private/Language/locallang_siteconfiguration_tca.xlf:site_route.staticText',
             'config' => [
                 'type' => 'text',
-                'eval' => 'required',
+                'required' => true,
                 'valuePicker' => [
                     'items' => [
                         ['LLL:EXT:backend/Resources/Private/Language/locallang_siteconfiguration_tca.xlf:site_route.staticText.example1', 'User-agent: *
 Disallow: /typo3/
 Disallow: /typo3_src/
-Allow: /typo3/sysext/frontend/Resources/Public/*
+Disallow: /fileadmin/user_upload/_temp_/importexport
+# Uncomment the following line if you have a sitemap enabled. Replace the domain with your own.
+# Sitemap: https://www.example.org/sitemap.xml
 '],
                     ],
                 ],
@@ -57,19 +62,25 @@ Allow: /typo3/sysext/frontend/Resources/Public/*
         ],
         'source' => [
             'label' => 'LLL:EXT:backend/Resources/Private/Language/locallang_siteconfiguration_tca.xlf:site_route.source',
+            'config' => [
+                'type' => 'link',
+                'required' => true,
+                'allowedTypes' => ['page', 'url', 'record', 'file'],
+            ],
+        ],
+        'asset' => [
+            'label' => 'LLL:EXT:backend/Resources/Private/Language/locallang_siteconfiguration_tca.xlf:site_route.asset',
             'config' => [
                 'type' => 'input',
-                'renderType' => 'inputLink',
-                'eval' => 'required',
-                'fieldControl' => [
-                    'linkPopup' => [
-                        'options' => [
-                            'blindLinkOptions' => 'mail,telephone,spec,folder',
-                        ]
-                    ]
+                'eval' => 'trim',
+                'required' => true,
+                'valuePicker' => [
+                    'items' => [
+                        ['LLL:EXT:backend/Resources/Private/Language/locallang_siteconfiguration_tca.xlf:site_route.asset.example', 'EXT:backend/Resources/Public/Icons/favicon.ico'],
+                    ],
                 ],
             ],
-        ]
+        ],
     ],
     'types' => [
         '1' => [
@@ -80,6 +91,9 @@ Allow: /typo3/sysext/frontend/Resources/Public/*
         ],
         'uri' => [
             'showitem' => 'route, type, source',
-        ]
+        ],
+        'asset' => [
+            'showitem' => 'route, type, asset',
+        ],
     ],
 ];
