@@ -34,34 +34,31 @@ class DumpAutoloadCommand extends Command
      */
     protected function configure()
     {
+        $this->setName('dumpautoload');
         $this->setDescription('Updates class loading information in non-composer mode.');
         $this->setHelp('This command is only needed during development. The extension manager takes care of creating or updating this info properly during extension (de-)activation.');
         $this->setAliases([
             'extensionmanager:extension:dumpclassloadinginformation',
-            'extension:dumpclassloadinginformation'
+            'extension:dumpclassloadinginformation',
         ]);
     }
 
     /**
      * This command is not needed in composer mode.
-     *
-     * @inheritdoc
      */
-    public function isEnabled()
+    public function isEnabled(): bool
     {
         return !Environment::isComposerMode();
     }
 
     /**
      * Dumps the class loading information
-     *
-     * @inheritdoc
      */
-    protected function execute(InputInterface $input, OutputInterface $output)
+    protected function execute(InputInterface $input, OutputInterface $output): int
     {
         $io = new SymfonyStyle($input, $output);
         ClassLoadingInformation::dumpClassLoadingInformation();
         $io->success('Class loading information has been updated.');
-        return 0;
+        return Command::SUCCESS;
     }
 }
