@@ -16,40 +16,37 @@
 namespace TYPO3\CMS\Core\Core;
 
 use Composer\Autoload\ClassLoader;
+use Composer\InstalledVersions;
 use Doctrine\Common\Annotations\AnnotationReader;
-use Doctrine\Common\Annotations\AnnotationRegistry;
 use Psr\Container\ContainerInterface;
 use Psr\EventDispatcher\EventDispatcherInterface;
+use Psr\Http\Message\ServerRequestInterface;
 use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
 use TYPO3\CMS\Core\Cache\Backend\BackendInterface;
 use TYPO3\CMS\Core\Cache\Backend\NullBackend;
 use TYPO3\CMS\Core\Cache\Backend\Typo3DatabaseBackend;
-use TYPO3\CMS\Core\Cache\CacheManager;
 use TYPO3\CMS\Core\Cache\Exception\InvalidBackendException;
 use TYPO3\CMS\Core\Cache\Exception\InvalidCacheException;
 use TYPO3\CMS\Core\Cache\Frontend\FrontendInterface;
 use TYPO3\CMS\Core\Cache\Frontend\PhpFrontend;
 use TYPO3\CMS\Core\Cache\Frontend\VariableFrontend;
 use TYPO3\CMS\Core\Configuration\ConfigurationManager;
-use TYPO3\CMS\Core\Database\TableConfigurationPostProcessingHookInterface;
+use TYPO3\CMS\Core\Configuration\Extension\ExtLocalconfFactory;
+use TYPO3\CMS\Core\Configuration\Extension\ExtTablesFactory;
+use TYPO3\CMS\Core\Configuration\Tca\TcaFactory;
+use TYPO3\CMS\Core\Core\Event\BootCompletedEvent;
 use TYPO3\CMS\Core\DependencyInjection\Cache\ContainerBackend;
 use TYPO3\CMS\Core\DependencyInjection\ContainerBuilder;
-use TYPO3\CMS\Core\Imaging\IconRegistry;
-use TYPO3\CMS\Core\IO\PharStreamWrapperInterceptor;
-use TYPO3\CMS\Core\Localization\LanguageService;
 use TYPO3\CMS\Core\Log\LogManager;
+use TYPO3\CMS\Core\Package\Cache\ComposerPackageArtifact;
+use TYPO3\CMS\Core\Package\Cache\PackageCacheInterface;
+use TYPO3\CMS\Core\Package\Cache\PackageStatesPackageCache;
 use TYPO3\CMS\Core\Package\FailsafePackageManager;
 use TYPO3\CMS\Core\Package\PackageManager;
-use TYPO3\CMS\Core\Page\PageRenderer;
+use TYPO3\CMS\Core\Schema\TcaSchemaFactory;
 use TYPO3\CMS\Core\Service\DependencyOrderingService;
 use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
-use TYPO3\CMS\Core\Utility\StringUtility;
-use TYPO3\PharStreamWrapper\Behavior;
-use TYPO3\PharStreamWrapper\Interceptor\ConjunctionInterceptor;
-use TYPO3\PharStreamWrapper\Interceptor\PharMetaDataInterceptor;
-use TYPO3\PharStreamWrapper\Manager;
-use TYPO3\PharStreamWrapper\PharStreamWrapper;
 
 /**
  * This class encapsulates bootstrap related methods.
@@ -70,19 +67,19 @@ class Bootstrap
      *
      * @param ClassLoader $classLoader an instance of the class loader
      * @param bool $failsafe true if no caching and a failsafe package manager should be used
-     * @return ContainerInterface
      */
     public static function init(
         ClassLoader $classLoader,
         bool $failsafe = false
     ): ContainerInterface {
-        $requestId = substr(md5(StringUtility::getUniqueId()), 0, 13);
+        $requestId = new RequestId();
 
         static::initializeClassLoader($classLoader);
         if (!Environment::isComposerMode() && ClassLoadingInformation::isClassLoadingInformationAvailable()) {
             ClassLoadingInformation::registerClassLoadingInformation();
         }
 
+        // @todo Remove output buffering in TYPO3 v14
         static::startOutputBuffering();
 
         $configurationManager = static::createConfigurationManager();
@@ -91,37 +88,30 @@ class Bootstrap
         }
         static::populateLocalConfiguration($configurationManager);
 
-        $logManager = new LogManager($requestId);
+        $logManager = new LogManager((string)$requestId);
         // LogManager is used by the core ErrorHandler (using GeneralUtility::makeInstance),
         // therefore we have to push the LogManager to GeneralUtility, in case there
         // happen errors before we call GeneralUtility::setContainer().
         GeneralUtility::setSingletonInstance(LogManager::class, $logManager);
 
         static::initializeErrorHandling();
-        static::initializeIO();
 
         $disableCaching = $failsafe ? true : false;
+        /** @var PhpFrontend $coreCache */
         $coreCache = static::createCache('core', $disableCaching);
+        $packageCache = static::createPackageCache($coreCache);
         $packageManager = static::createPackageManager(
             $failsafe ? FailsafePackageManager::class : PackageManager::class,
-            $coreCache
+            $packageCache
         );
 
-        // Push PackageManager instance to ExtensionManagementUtility
-        // Should be fetched through the container (later) but currently a PackageManager
-        // singleton instance is required by PackageManager->activePackageDuringRuntime
-        GeneralUtility::setSingletonInstance(PackageManager::class, $packageManager);
-        ExtensionManagementUtility::setPackageManager($packageManager);
-        static::initializeRuntimeActivatedPackagesFromConfiguration($packageManager);
-
         static::setDefaultTimezone();
         static::setMemoryLimit();
 
-        $assetsCache = static::createCache('assets', $disableCaching);
         $dependencyInjectionContainerCache = static::createCache('di');
 
         $bootState = new \stdClass();
-        $bootState->done = false;
+        $bootState->complete = false;
         $bootState->cacheDisabled = $disableCaching;
 
         $builder = new ContainerBuilder([
@@ -129,9 +119,9 @@ class Bootstrap
             ApplicationContext::class => Environment::getContext(),
             ConfigurationManager::class => $configurationManager,
             LogManager::class => $logManager,
+            RequestId::class => $requestId,
             'cache.di' => $dependencyInjectionContainerCache,
             'cache.core' => $coreCache,
-            'cache.assets' => $assetsCache,
             PackageManager::class => $packageManager,
 
             // @internal
@@ -144,24 +134,27 @@ class Bootstrap
         // makeInstance() method creates classes using the container from now on.
         GeneralUtility::setContainer($container);
 
-        // Reset singleton instances in order for GeneralUtility::makeInstance() to use
-        // ContainerInterface->get() for early services from now on.
+        // Reset LogManager singleton instance in order for GeneralUtility::makeInstance()
+        // to proxy LogManager retrieval to ContainerInterface->get() from now on.
         GeneralUtility::removeSingletonInstance(LogManager::class, $logManager);
-        GeneralUtility::removeSingletonInstance(PackageManager::class, $packageManager);
+
+        // Push PackageManager instance to ExtensionManagementUtility
+        ExtensionManagementUtility::setPackageManager($packageManager);
 
         if ($failsafe) {
-            $bootState->done = true;
+            $bootState->complete = true;
             return $container;
         }
 
-        IconRegistry::setCache($assetsCache);
-        PageRenderer::setCache($assetsCache);
-        ExtensionManagementUtility::setEventDispatcher($container->get(EventDispatcherInterface::class));
-        static::loadTypo3LoadedExtAndExtLocalconf(true, $coreCache);
+        $eventDispatcher = $container->get(EventDispatcherInterface::class);
+        $tcaFactory = $container->get(TcaFactory::class);
+        $container->get(ExtLocalconfFactory::class)->load();
         static::unsetReservedGlobalVariables();
-        $bootState->done = true;
-        static::loadBaseTca(true, $coreCache);
+        $GLOBALS['TCA'] = $tcaFactory->get();
         static::checkEncryptionKey();
+        $bootState->complete = true;
+        $container->get(TcaSchemaFactory::class)->load($GLOBALS['TCA']);
+        $eventDispatcher->dispatch(new BootCompletedEvent(true));
 
         return $container;
     }
@@ -183,14 +176,10 @@ class Bootstrap
      *
      * Script execution will be aborted if something fails here.
      *
-     * @throws \RuntimeException when TYPO3_REQUESTTYPE was not set before, setRequestType() needs to be called before
      * @internal This is not a public API method, do not use in own extensions
      */
     public static function baseSetup()
     {
-        if (!defined('TYPO3_REQUESTTYPE')) {
-            throw new \RuntimeException('No Request Type was set, TYPO3 does not know in which context it is run.', 1450561838);
-        }
         if (!Environment::isComposerMode() && ClassLoadingInformation::isClassLoadingInformationAvailable()) {
             ClassLoadingInformation::registerClassLoadingInformation();
         }
@@ -206,9 +195,6 @@ class Bootstrap
     {
         ClassLoadingInformation::setClassLoader($classLoader);
 
-        /** @see initializeAnnotationRegistry */
-        AnnotationRegistry::registerLoader([$classLoader, 'loadClass']);
-
         // Annotations used in unit tests
         AnnotationReader::addGlobalIgnoredName('test');
 
@@ -218,17 +204,68 @@ class Bootstrap
     }
 
     /**
-     * checks if LocalConfiguration.php or PackageStates.php is missing,
-     * used to see if a redirect to the install tool is needed
+     * checks if config/system/settings.php or PackageStates.php is missing,
+     * used to see if a redirect to the installer is needed
+     *
+     * All file_exists checks are delayed as far as possible to avoid I/O impact
      *
-     * @param ConfigurationManager $configurationManager
      * @return bool TRUE when the essential configuration is available, otherwise FALSE
      * @internal This is not a public API method, do not use in own extensions
      */
-    protected static function checkIfEssentialConfigurationExists(ConfigurationManager $configurationManager): bool
+    public static function checkIfEssentialConfigurationExists(ConfigurationManager $configurationManager): bool
     {
-        return file_exists($configurationManager->getLocalConfigurationFileLocation())
-            && file_exists(Environment::getLegacyConfigPath() . '/PackageStates.php');
+        if (!Environment::isComposerMode()
+            && !file_exists(Environment::getLegacyConfigPath() . '/PackageStates.php')
+        ) {
+            // Early return in case system is not properly set up
+            return false;
+        }
+
+        $systemConfigurationPath = $configurationManager->getSystemConfigurationFileLocation();
+        $additionalConfigurationPath = $configurationManager->getAdditionalConfigurationFileLocation();
+
+        $systemConfigurationFileExists = file_exists($systemConfigurationPath);
+        $additionalConfigurationFileExists = file_exists($additionalConfigurationPath);
+        if ($systemConfigurationFileExists && $additionalConfigurationFileExists) {
+            // We have a complete configuration, off we go
+            return true;
+        }
+
+        // If system configuration file exists and no legacy additional configuration is present, we are good
+        $legacyAdditionConfigurationPath = Environment::getLegacyConfigPath() . '/AdditionalConfiguration.php';
+        $legacyAdditionalConfigurationFileExists = file_exists($legacyAdditionConfigurationPath);
+        if ($systemConfigurationFileExists && !$legacyAdditionalConfigurationFileExists) {
+            return true;
+        }
+
+        // @deprecated All code below is deprecated and can be removed with TYPO3 v15.0 (or later as
+        //              it does not hurt to keep this migration for now) and replaced with `return false;`
+
+        // All other cases will probably need some migration work
+        $migrated = false;
+
+        // In case no system configuration file exists at this point, check for the legacy "LocalConfiguration"
+        // file. If it exists, move it to the new location. Otherwise, the system is not complete.
+        if (!$systemConfigurationFileExists) {
+            $legacyLocalConfigurationPath = $configurationManager->getLocalConfigurationFileLocation();
+            $legacySystemConfigurationFileExists = file_exists($legacyLocalConfigurationPath);
+            if ($legacySystemConfigurationFileExists) {
+                mkdir(dirname($systemConfigurationPath), 02775, true);
+                rename($legacyLocalConfigurationPath, $systemConfigurationPath);
+                $migrated = true;
+            } else {
+                // Directly return as essential system configuration does not exist
+                return false;
+            }
+        }
+        // In case no additional configuration file exists at this point, check for the legacy
+        // "AdditionalConfiguration" file. If it exists, move it to the new location as well.
+        if (!$additionalConfigurationFileExists && $legacyAdditionalConfigurationFileExists) {
+            rename($legacyAdditionConfigurationPath, $additionalConfigurationPath);
+            $migrated = true;
+        }
+
+        return $migrated;
     }
 
     /**
@@ -236,58 +273,39 @@ class Bootstrap
      * provided by the packages.
      *
      * @param string $packageManagerClassName Define an alternative package manager implementation (usually for the installer)
-     * @param FrontendInterface $coreCache
-     * @return PackageManager
      * @internal This is not a public API method, do not use in own extensions
      */
-    public static function createPackageManager($packageManagerClassName, FrontendInterface $coreCache): PackageManager
+    public static function createPackageManager($packageManagerClassName, PackageCacheInterface $packageCache): PackageManager
     {
         $dependencyOrderingService = GeneralUtility::makeInstance(DependencyOrderingService::class);
-        /** @var \TYPO3\CMS\Core\Package\PackageManager $packageManager */
+        /** @var PackageManager $packageManager */
         $packageManager = new $packageManagerClassName($dependencyOrderingService);
-        $packageManager->injectCoreCache($coreCache);
+        $packageManager->setPackageCache($packageCache);
         $packageManager->initialize();
 
         return $packageManager;
     }
 
     /**
-     * Activates a package during runtime. This is used in AdditionalConfiguration.php
-     * to enable extensions under conditions.
-     *
-     * @param PackageManager $packageManager
+     * @internal
      */
-    protected static function initializeRuntimeActivatedPackagesFromConfiguration(PackageManager $packageManager)
+    public static function createPackageCache(FrontendInterface $coreCache): PackageCacheInterface
     {
-        $packages = $GLOBALS['TYPO3_CONF_VARS']['EXT']['runtimeActivatedPackages'] ?? [];
-        if (!empty($packages)) {
-            trigger_error('Support for runtime activated packages will be removed in TYPO3 v11.0.', E_USER_DEPRECATED);
-            foreach ($packages as $runtimeAddedPackageKey) {
-                $packageManager->activatePackageDuringRuntime($runtimeAddedPackageKey);
-            }
+        if (!Environment::isComposerMode()) {
+            return new PackageStatesPackageCache(Environment::getLegacyConfigPath() . '/PackageStates.php', $coreCache);
         }
-    }
 
-    /**
-     * Load ext_localconf of extensions
-     *
-     * @param bool $allowCaching
-     * @param FrontendInterface $coreCache
-     * @internal This is not a public API method, do not use in own extensions
-     */
-    public static function loadTypo3LoadedExtAndExtLocalconf($allowCaching = true, FrontendInterface $coreCache = null)
-    {
-        if ($allowCaching) {
-            $coreCache = $coreCache ?? GeneralUtility::makeInstance(CacheManager::class)->getCache('core');
+        $composerInstallersPath = InstalledVersions::getInstallPath('typo3/cms-composer-installers');
+        if ($composerInstallersPath === null) {
+            throw new \RuntimeException('Package "typo3/cms-composer-installers" not found. Replacing the package is not allowed. Fork the package instead and pull in the fork with the same name.', 1636145677);
         }
-        ExtensionManagementUtility::loadExtLocalconf($allowCaching, $coreCache);
+
+        return new ComposerPackageArtifact(dirname($composerInstallersPath));
     }
 
     /**
      * We need an early instance of the configuration manager.
      * Since makeInstance relies on the object configuration, we create it here with new instead.
-     *
-     * @return ConfigurationManager
      */
     public static function createConfigurationManager(): ConfigurationManager
     {
@@ -298,7 +316,6 @@ class Bootstrap
      * We need an early instance of the configuration manager.
      * Since makeInstance relies on the object configuration, we create it here with new instead.
      *
-     * @param ConfigurationManager $configurationManager
      * @internal This is not a public API method, do not use in own extensions
      */
     protected static function populateLocalConfiguration(ConfigurationManager $configurationManager)
@@ -315,11 +332,14 @@ class Bootstrap
      *
      * @param string $identifier
      * @param bool $disableCaching
-     * @return FrontendInterface
+     * @param class-string<BackendInterface>|null $enforcedCacheBackend
      * @internal
      */
-    public static function createCache(string $identifier, bool $disableCaching = false): FrontendInterface
-    {
+    public static function createCache(
+        string $identifier,
+        bool $disableCaching = false,
+        ?string $enforcedCacheBackend = null
+    ): FrontendInterface {
         $cacheConfigurations = $GLOBALS['TYPO3_CONF_VARS']['SYS']['caching']['cacheConfigurations'] ?? [];
         $cacheConfigurations['di']['frontend'] = PhpFrontend::class;
         $cacheConfigurations['di']['backend'] = ContainerBackend::class;
@@ -327,7 +347,7 @@ class Bootstrap
         $configuration = $cacheConfigurations[$identifier] ?? [];
 
         $frontend = $configuration['frontend'] ?? VariableFrontend::class;
-        $backend = $configuration['backend'] ?? Typo3DatabaseBackend::class;
+        $backend = $enforcedCacheBackend ?? $configuration['backend'] ?? Typo3DatabaseBackend::class;
         $options = $configuration['options'] ?? [];
 
         if ($disableCaching) {
@@ -380,9 +400,6 @@ class Bootstrap
      */
     protected static function initializeErrorHandling()
     {
-        static::initializeBasicErrorReporting();
-        $displayErrors = 0;
-        $exceptionHandlerClassName = null;
         $productionExceptionHandlerClassName = $GLOBALS['TYPO3_CONF_VARS']['SYS']['productionExceptionHandler'];
         $debugExceptionHandlerClassName = $GLOBALS['TYPO3_CONF_VARS']['SYS']['debugExceptionHandler'];
 
@@ -407,13 +424,13 @@ class Bootstrap
                 $displayErrors = 1;
                 break;
             default:
-                if (!(TYPO3_REQUESTTYPE & TYPO3_REQUESTTYPE_INSTALL)) {
-                    // Throw exception if an invalid option is set.
-                    throw new \RuntimeException(
-                        'The option $TYPO3_CONF_VARS[SYS][displayErrors] is not set to "-1", "0" or "1".',
-                        1476046290
-                    );
-                }
+                // Throw exception if an invalid option is set. A default for displayErrors is set
+                // in very early install tool, coming from DefaultConfiguration.php. It is safe here
+                // to just throw if there is no value for whatever reason.
+                throw new \RuntimeException(
+                    'The option $TYPO3_CONF_VARS[SYS][displayErrors] is not set to "-1", "0" or "1".',
+                    1476046290
+                );
         }
         @ini_set('display_errors', (string)$displayErrors);
 
@@ -434,46 +451,6 @@ class Bootstrap
         }
     }
 
-    /**
-     * Initialize basic error reporting.
-     *
-     * There are a lot of extensions that have no strict / notice / deprecated free
-     * ext_localconf or ext_tables. Since the final error reporting must be set up
-     * after those extension files are read, a default configuration is needed to
-     * suppress error reporting meanwhile during further bootstrap.
-     *
-     * Please note: if you comment out this code, TYPO3 would never set any error reporting
-     * which would need to have TYPO3 Core and ALL used extensions to be notice free and deprecation
-     * free. However, commenting this out and running functional and acceptance tests shows what
-     * needs to be done to make TYPO3 Core mostly notice-free (unit tests do not execute this code here).
-     */
-    protected static function initializeBasicErrorReporting(): void
-    {
-        // Core should be notice free at least until this point ...
-        error_reporting(E_ALL & ~(E_STRICT | E_NOTICE | E_DEPRECATED));
-    }
-
-    /**
-     * Initializes IO and stream wrapper related behavior.
-     */
-    protected static function initializeIO()
-    {
-        if (in_array('phar', stream_get_wrappers())) {
-            // destroy and re-initialize PharStreamWrapper for TYPO3 core
-            Manager::destroy();
-            Manager::initialize(
-                (new Behavior())
-                    ->withAssertion(new ConjunctionInterceptor([
-                        new PharStreamWrapperInterceptor(),
-                        new PharMetaDataInterceptor()
-                    ]))
-            );
-
-            stream_wrapper_unregister('phar');
-            stream_wrapper_register('phar', PharStreamWrapper::class);
-        }
-    }
-
     /**
      * Set PHP memory limit depending on value of
      * $GLOBALS['TYPO3_CONF_VARS']['SYS']['setMemoryLimit']
@@ -493,32 +470,8 @@ class Bootstrap
      */
     public static function unsetReservedGlobalVariables()
     {
-        unset($GLOBALS['PAGES_TYPES']);
         unset($GLOBALS['TCA']);
-        unset($GLOBALS['TBE_MODULES']);
-        unset($GLOBALS['TBE_STYLES']);
         unset($GLOBALS['BE_USER']);
-        // Those set otherwise:
-        unset($GLOBALS['TBE_MODULES_EXT']);
-        unset($GLOBALS['TCA_DESCR']);
-        unset($GLOBALS['LOCAL_LANG']);
-    }
-
-    /**
-     * Load $TCA
-     *
-     * This will mainly set up $TCA through extMgm API.
-     *
-     * @param bool $allowCaching True, if loading TCA from cache is allowed
-     * @param FrontendInterface $coreCache
-     * @internal This is not a public API method, do not use in own extensions
-     */
-    public static function loadBaseTca(bool $allowCaching = true, FrontendInterface $coreCache = null)
-    {
-        if ($allowCaching) {
-            $coreCache = $coreCache ?? GeneralUtility::makeInstance(CacheManager::class)->getCache('core');
-        }
-        ExtensionManagementUtility::loadBaseTca($allowCaching, $coreCache);
     }
 
     /**
@@ -542,81 +495,40 @@ class Bootstrap
      *
      * @param bool $allowCaching True, if reading compiled ext_tables file from cache is allowed
      * @internal This is not a public API method, do not use in own extensions
+     * @todo: It would be better to remove this method and use the factory directly.
+     *        Needs a pre-patch in testing-framework.
      */
-    public static function loadExtTables(bool $allowCaching = true)
+    public static function loadExtTables(bool $allowCaching = true, ?FrontendInterface $coreCache = null)
     {
-        ExtensionManagementUtility::loadExtTables($allowCaching);
-        static::runExtTablesPostProcessingHooks();
-    }
-
-    /**
-     * Check for registered ext tables hooks and run them
-     *
-     * @throws \UnexpectedValueException
-     */
-    protected static function runExtTablesPostProcessingHooks()
-    {
-        foreach ($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['GLOBAL']['extTablesInclusion-PostProcessing'] ?? [] as $className) {
-            /** @var \TYPO3\CMS\Core\Database\TableConfigurationPostProcessingHookInterface $hookObject */
-            $hookObject = GeneralUtility::makeInstance($className);
-            if (!$hookObject instanceof TableConfigurationPostProcessingHookInterface) {
-                throw new \UnexpectedValueException(
-                    '$hookObject "' . $className . '" must implement interface TYPO3\\CMS\\Core\\Database\\TableConfigurationPostProcessingHookInterface',
-                    1320585902
-                );
-            }
-            $hookObject->processData();
+        $container = GeneralUtility::getContainer();
+        if ($allowCaching) {
+            $container->get(ExtTablesFactory::class)->load();
+        } else {
+            $container->get(ExtTablesFactory::class)->loadUncached();
         }
     }
 
-    /**
-     * Initialize the Routing for the TYPO3 Backend
-     * Loads all routes registered inside all packages and stores them inside the Router
-     *
-     * @internal This is not a public API method, do not use in own extensions
-     * @deprecated this does not do anything anymore, as TYPO3's dependency injection already loads the routes on demand.
-     */
-    public static function initializeBackendRouter()
-    {
-        // TODO: Once typo3/testing-framework is adapted, this code can be dropped / deprecated. As DI is already
-        // loading all routes on demand, this method is not needed anymore.
-    }
-
     /**
      * Initialize backend user object in globals
      *
      * @param string $className usually \TYPO3\CMS\Core\Authentication\BackendUserAuthentication::class but can be used for CLI
-     * @internal This is not a public API method, do not use in own extensions
+     * @param ServerRequestInterface|null $request
      */
-    public static function initializeBackendUser($className = BackendUserAuthentication::class)
+    public static function initializeBackendUser($className = BackendUserAuthentication::class, ?ServerRequestInterface $request = null)
     {
-        /** @var \TYPO3\CMS\Core\Authentication\BackendUserAuthentication $backendUser */
+        /** @var BackendUserAuthentication $backendUser */
         $backendUser = GeneralUtility::makeInstance($className);
         // The global must be available very early, because methods below
         // might trigger code which relies on it. See: #45625
         $GLOBALS['BE_USER'] = $backendUser;
-        $backendUser->start();
+        $backendUser->start($request);
     }
 
     /**
      * Initializes and ensures authenticated access
-     *
-     * @internal This is not a public API method, do not use in own extensions
-     * @param bool $proceedIfNoUserIsLoggedIn if set to TRUE, no forced redirect to the login page will be done
      */
-    public static function initializeBackendAuthentication($proceedIfNoUserIsLoggedIn = false)
+    public static function initializeBackendAuthentication()
     {
-        $GLOBALS['BE_USER']->backendCheckLogin($proceedIfNoUserIsLoggedIn);
-    }
-
-    /**
-     * Initialize language object
-     *
-     * @internal This is not a public API method, do not use in own extensions
-     */
-    public static function initializeLanguageObject()
-    {
-        /** @var \TYPO3\CMS\Core\Localization\LanguageService $GLOBALS['LANG'] */
-        $GLOBALS['LANG'] = LanguageService::createFromUserPreferences($GLOBALS['BE_USER']);
+        $GLOBALS['BE_USER']->backendCheckLogin();
     }
 }
