@@ -1,7 +1,6 @@
 <?php
-declare(strict_types = 1);
 
-namespace TYPO3\CMS\Core\Database\Schema\Parser\AST\DataType;
+declare(strict_types=1);
 
 /*
  * This file is part of the TYPO3 CMS project.
@@ -16,16 +15,15 @@ namespace TYPO3\CMS\Core\Database\Schema\Parser\AST\DataType;
  * The TYPO3 project - inspiring people to share!
  */
 
+namespace TYPO3\CMS\Core\Database\Schema\Parser\AST\DataType;
+
 /**
  * Node representing the BIT SQL column type
+ *
+ * @internal
  */
-class BitDataType extends AbstractDataType
+final class BitDataType extends AbstractDataType
 {
-    /**
-     * BitDataType constructor.
-     *
-     * @param int $length
-     */
     public function __construct(int $length)
     {
         $this->length = $length;
