@@ -29,36 +29,32 @@ final class PseudoServiceProvider extends AbstractServiceProvider
      */
     private $package;
 
-    /**
-     * @param PackageInterface $package
-     */
     public function __construct(PackageInterface $package)
     {
         $this->package = $package;
     }
 
-    /**
-     * @return string
-     */
     protected static function getPackagePath(): string
     {
         throw new \BadMethodCallException('PseudoServiceProvider does not support the getPackagePath() method.', 1562354465);
     }
 
-    /**
-     * @return array
-     */
+    protected static function getPackageName(): string
+    {
+        throw new \BadMethodCallException('PseudoServiceProvider does not support the getPackageName() method.', 1643372902);
+    }
+
     public function getFactories(): array
     {
         return [];
     }
 
-    /**
-     * @return array
-     */
     public function getExtensions(): array
     {
         $packagePath = $this->package->getPackagePath();
+        // Fallback to empty string if dealing with an extension in non-composer mode
+        // that still does not provide composer.json.
+        $packageName = $this->package->getValueFromComposerManifest('name') ?? '';
         $extensions = parent::getExtensions();
 
         // The static configure*() methods in AbstractServiceProvider use the
@@ -69,9 +65,10 @@ final class PseudoServiceProvider extends AbstractServiceProvider
         // AbstractServiceProvider configure methods are aware of this and
         // provide an optional third parameter which is forwarded as
         // dynamic path to getPackagePath().
+        // Same logic for $packageName.
         foreach ($extensions as $serviceName => $previousCallable) {
-            $extensions[$serviceName] = function (ContainerInterface $container, $value) use ($previousCallable, $packagePath) {
-                return ($previousCallable)($container, $value, $packagePath);
+            $extensions[$serviceName] = static function (ContainerInterface $container, $value) use ($previousCallable, $packagePath, $packageName) {
+                return ($previousCallable)($container, $value, $packagePath, $packageName);
             };
         }
 
