@@ -64,7 +64,7 @@ class DriverRegistry implements SingletonInterface
             throw new \InvalidArgumentException('Class ' . $className . ' does not exist.', 1314979197);
         }
 
-        if (!in_array(DriverInterface::class, class_implements($className), true)) {
+        if (!in_array(DriverInterface::class, class_implements($className) ?: [], true)) {
             throw new \InvalidArgumentException('Driver ' . $className . ' needs to implement the DriverInterface.', 1387619575);
         }
         if ($shortName === '') {
@@ -82,7 +82,7 @@ class DriverRegistry implements SingletonInterface
             'class' => $className,
             'shortName' => $shortName,
             'label' => $label,
-            'flexFormDS' => $flexFormDataStructurePathAndFilename
+            'flexFormDS' => $flexFormDataStructurePathAndFilename,
         ];
         return true;
     }
@@ -97,7 +97,7 @@ class DriverRegistry implements SingletonInterface
         $configurationFieldConfig = &$GLOBALS['TCA']['sys_file_storage']['columns']['configuration']['config'];
         foreach ($this->driverConfigurations as $driver) {
             $label = $driver['label'] ?: $driver['class'];
-            $driverFieldConfig['items'][$driver['shortName']] = [$label, $driver['shortName']];
+            $driverFieldConfig['items'][$driver['shortName']] = ['label' => $label, 'value' => $driver['shortName']];
             if ($driver['flexFormDS']) {
                 $configurationFieldConfig['ds'][$driver['shortName']] = $driver['flexFormDS'];
             }
