@@ -23,17 +23,9 @@ namespace TYPO3\CMS\Core\Resource\Event;
  * Example: If a file is marked as missing, listeners can try to recover a file. This can happen on specific setups
  * where editors also work via FTP.
  */
-final class AfterFileMarkedAsMissingEvent
+final readonly class AfterFileMarkedAsMissingEvent
 {
-    /**
-     * @var int
-     */
-    private $fileUid;
-
-    public function __construct(int $fileUid)
-    {
-        $this->fileUid = $fileUid;
-    }
+    public function __construct(private int $fileUid) {}
 
     public function getFileUid(): int
     {
