@@ -15,7 +15,12 @@
 
 namespace TYPO3\CMS\Core\Resource;
 
+use Psr\Http\Message\UploadedFileInterface;
+use TYPO3\CMS\Core\Resource\Enum\DuplicationBehavior;
+use TYPO3\CMS\Core\Resource\Exception\ExistingTargetFileNameException;
+use TYPO3\CMS\Core\Resource\Exception\ExistingTargetFolderException;
 use TYPO3\CMS\Core\Resource\Exception\InsufficientFolderAccessPermissionsException;
+use TYPO3\CMS\Core\Resource\Exception\InsufficientFolderWritePermissionsException;
 use TYPO3\CMS\Core\Resource\Exception\ResourcePermissionsUnavailableException;
 use TYPO3\CMS\Core\Resource\Search\FileSearchDemand;
 use TYPO3\CMS\Core\Resource\Search\Result\FileSearchResultInterface;
@@ -35,6 +40,16 @@ use TYPO3\CMS\Core\Utility\PathUtility;
  */
 class Folder implements FolderInterface
 {
+    /**
+     * Modes for filter usage in getFiles()/getFolders()
+     */
+    public const FILTER_MODE_NO_FILTERS = 0;
+    // Merge local filters into storage's filters
+    public const FILTER_MODE_USE_OWN_AND_STORAGE_FILTERS = 1;
+    // Only use the filters provided by the storage
+    public const FILTER_MODE_USE_STORAGE_FILTERS = 2;
+    // Only use the filters provided by the current class
+    public const FILTER_MODE_USE_OWN_FILTERS = 3;
     /**
      * The storage this folder belongs to.
      *
@@ -65,21 +80,9 @@ class Folder implements FolderInterface
      */
     protected $fileAndFolderNameFilters = [];
 
-    /**
-     * Modes for filter usage in getFiles()/getFolders()
-     */
-    const FILTER_MODE_NO_FILTERS = 0;
-    // Merge local filters into storage's filters
-    const FILTER_MODE_USE_OWN_AND_STORAGE_FILTERS = 1;
-    // Only use the filters provided by the storage
-    const FILTER_MODE_USE_STORAGE_FILTERS = 2;
-    // Only use the filters provided by the current class
-    const FILTER_MODE_USE_OWN_FILTERS = 3;
-
     /**
      * Initialization of the folder
      *
-     * @param ResourceStorage $storage
      * @param string $identifier
      * @param string $name
      */
@@ -90,12 +93,7 @@ class Folder implements FolderInterface
         $this->name = $name;
     }
 
-    /**
-     * Returns the name of this folder.
-     *
-     * @return string
-     */
-    public function getName()
+    public function getName(): string
     {
         return $this->name;
     }
@@ -110,7 +108,7 @@ class Folder implements FolderInterface
     public function getReadablePath($rootId = null)
     {
         if ($rootId === null) {
-            // Find first matching filemount and use that as root
+            // Find first matching file mount and use that as root
             foreach ($this->storage->getFileMounts() as $fileMount) {
                 if ($this->storage->isWithinFolder($fileMount['folder'], $this)) {
                     $rootId = $fileMount['folder']->getIdentifier();
@@ -145,12 +143,7 @@ class Folder implements FolderInterface
         $this->name = $name;
     }
 
-    /**
-     * Returns the storage this folder belongs to.
-     *
-     * @return ResourceStorage
-     */
-    public function getStorage()
+    public function getStorage(): ResourceStorage
     {
         return $this->storage;
     }
@@ -159,19 +152,14 @@ class Folder implements FolderInterface
      * Returns the path of this folder inside the storage. It depends on the
      * type of storage whether this is a real path or just some unique identifier.
      *
-     * @return string
+     * @return non-empty-string
      */
-    public function getIdentifier()
+    public function getIdentifier(): string
     {
         return $this->identifier;
     }
 
-    /**
-     * Get hashed identifier
-     *
-     * @return string
-     */
-    public function getHashedIdentifier()
+    public function getHashedIdentifier(): string
     {
         return $this->storage->hashFileIdentifier($this->identifier);
     }
@@ -193,12 +181,11 @@ class Folder implements FolderInterface
      * WARNING: Access to the folder may be restricted by further means, e.g. some
      * web-based authentication. You have to take care of this yourself.
      *
-     * @param bool $relativeToCurrentScript Determines whether the URL returned should be relative to the current script, in case it is relative at all (only for the LocalDriver)
      * @return string|null NULL if file is missing or deleted, the generated url otherwise
      */
-    public function getPublicUrl($relativeToCurrentScript = false)
+    public function getPublicUrl()
     {
-        return $this->getStorage()->getPublicUrl($this, $relativeToCurrentScript);
+        return $this->getStorage()->getPublicUrl($this);
     }
 
     /**
@@ -240,9 +227,7 @@ class Folder implements FolderInterface
      * Returns a file search result based on the given demand.
      * The result also includes matches in meta data fields that are defined in TCA.
      *
-     * @param FileSearchDemand $searchDemand
      * @param int $filterMode The filter mode to use for the found files
-     * @return FileSearchResultInterface
      */
     public function searchFiles(FileSearchDemand $searchDemand, int $filterMode = self::FILTER_MODE_USE_OWN_AND_STORAGE_FILTERS): FileSearchResultInterface
     {
@@ -257,7 +242,6 @@ class Folder implements FolderInterface
      * Returns amount of all files within this folder, optionally filtered by
      * the given pattern
      *
-     * @param array $filterMethods
      * @param bool $recursive
      * @return int
      * @throws Exception\InsufficientFolderAccessPermissionsException
@@ -270,11 +254,9 @@ class Folder implements FolderInterface
     /**
      * Returns the object for a subfolder of the current folder, if it exists.
      *
-     * @param string $name Name of the subfolder
-     * @return Folder
      * @throws \InvalidArgumentException
      */
-    public function getSubfolder($name)
+    public function getSubfolder(string $name): Folder
     {
         if (!$this->storage->hasFolderInFolder($name, $this)) {
             throw new \InvalidArgumentException('Folder "' . $name . '" does not exist in "' . $this->identifier . '"', 1329836110);
@@ -283,15 +265,13 @@ class Folder implements FolderInterface
     }
 
     /**
-     * Returns a list of subfolders
-     *
      * @param int $start The item to start at
      * @param int $numberOfItems The number of items to return
      * @param int $filterMode The filter mode to use for the filelist.
      * @param bool $recursive
-     * @return Folder[]
+     * @phpstan-return array<array-key, Folder>
      */
-    public function getSubfolders($start = 0, $numberOfItems = 0, $filterMode = self::FILTER_MODE_USE_OWN_AND_STORAGE_FILTERS, $recursive = false)
+    public function getSubfolders($start = 0, $numberOfItems = 0, $filterMode = self::FILTER_MODE_USE_OWN_AND_STORAGE_FILTERS, $recursive = false): array
     {
         [$backedUpFilters, $useFilters] = $this->prepareFiltersInStorage($filterMode);
         $folderObjects = $this->storage->getFoldersInFolder($this, $start, $numberOfItems, $useFilters, $recursive);
@@ -305,34 +285,54 @@ class Folder implements FolderInterface
      *
      * @param string $localFilePath
      * @param string $fileName
-     * @param string $conflictMode a value of the \TYPO3\CMS\Core\Resource\DuplicationBehavior enumeration
+     * @param string|DuplicationBehavior $conflictMode
      * @return File The file object
+     * @throws ExistingTargetFileNameException
+     * @todo change $conflictMode parameter type to DuplicationBehavior in TYPO3 v14.0
      */
     public function addFile($localFilePath, $fileName = null, $conflictMode = DuplicationBehavior::CANCEL)
     {
         $fileName = $fileName ?: PathUtility::basename($localFilePath);
+
+        if (!$conflictMode instanceof DuplicationBehavior) {
+            trigger_error(
+                'Using the non-native enumeration TYPO3\CMS\Core\Resource\DuplicationBehavior in Folder->addFile()'
+                . ' will stop working in TYPO3 v14.0. Use native TYPO3\CMS\Core\Resource\Enum\DuplicationBehavior instead.',
+                E_USER_DEPRECATED
+            );
+            $conflictMode = DuplicationBehavior::tryFrom($conflictMode) ?? DuplicationBehavior::getDefaultDuplicationBehaviour();
+        }
+
         return $this->storage->addFile($localFilePath, $this, $fileName, $conflictMode);
     }
 
     /**
      * Adds an uploaded file into the Storage.
      *
-     * @param array $uploadedFileData contains information about the uploaded file given by $_FILES['file1']
-     * @param string $conflictMode a value of the \TYPO3\CMS\Core\Resource\DuplicationBehavior enumeration
-     * @return File The file object
+     * @param array|UploadedFileInterface $uploadedFileData Information about the uploaded file given by $_FILES['file1']
+     *                                                      or a PSR-7 UploadedFileInterface object
+     * @param string|DuplicationBehavior $conflictMode
+     * @return FileInterface The file object
+     * @todo change $conflictMode parameter type to DuplicationBehavior in TYPO3 v14.0
      */
-    public function addUploadedFile(array $uploadedFileData, $conflictMode = DuplicationBehavior::CANCEL)
+    public function addUploadedFile(array|UploadedFileInterface $uploadedFileData, $conflictMode = DuplicationBehavior::CANCEL)
     {
-        return $this->storage->addUploadedFile($uploadedFileData, $this, $uploadedFileData['name'], $conflictMode);
+        if (!$conflictMode instanceof DuplicationBehavior) {
+            trigger_error(
+                'Using the non-native enumeration TYPO3\CMS\Core\Resource\DuplicationBehavior in Folder->addUploadedFile()'
+                . ' will stop working in TYPO3 v14.0. Use native TYPO3\CMS\Core\Resource\Enum\DuplicationBehavior instead.',
+                E_USER_DEPRECATED
+            );
+            $conflictMode = DuplicationBehavior::tryFrom($conflictMode) ?? DuplicationBehavior::getDefaultDuplicationBehaviour();
+        }
+
+        return $this->storage->addUploadedFile($uploadedFileData, $this, null, $conflictMode);
     }
 
     /**
      * Renames this folder.
-     *
-     * @param string $newName
-     * @return Folder
      */
-    public function rename($newName)
+    public function rename(string $newName): self
     {
         return $this->storage->renameFolder($this, $newName);
     }
@@ -341,9 +341,8 @@ class Folder implements FolderInterface
      * Deletes this folder from its storage. This also means that this object becomes useless.
      *
      * @param bool $deleteRecursively
-     * @return bool TRUE if deletion succeeded
      */
-    public function delete($deleteRecursively = true)
+    public function delete($deleteRecursively = true): bool
     {
         return $this->storage->deleteFolder($this, $deleteRecursively);
     }
@@ -364,6 +363,8 @@ class Folder implements FolderInterface
      *
      * @param string $folderName
      * @return Folder The new folder object
+     * @throws ExistingTargetFolderException
+     * @throws InsufficientFolderWritePermissionsException
      */
     public function createFolder($folderName)
     {
@@ -375,11 +376,21 @@ class Folder implements FolderInterface
      *
      * @param Folder $targetFolder Target folder to copy to.
      * @param string $targetFolderName an optional destination fileName
-     * @param string $conflictMode a value of the \TYPO3\CMS\Core\Resource\DuplicationBehavior enumeration
+     * @param string|DuplicationBehavior $conflictMode
      * @return Folder New (copied) folder object.
+     * @todo change $conflictMode parameter type to DuplicationBehavior in TYPO3 v14.0
      */
     public function copyTo(Folder $targetFolder, $targetFolderName = null, $conflictMode = DuplicationBehavior::RENAME)
     {
+        if (!$conflictMode instanceof DuplicationBehavior) {
+            trigger_error(
+                'Using the non-native enumeration TYPO3\CMS\Core\Resource\DuplicationBehavior in Folder->copyTo()'
+                . ' will stop working in TYPO3 v14.0. Use native TYPO3\CMS\Core\Resource\Enum\DuplicationBehavior instead.',
+                E_USER_DEPRECATED
+            );
+            $conflictMode = DuplicationBehavior::tryFrom($conflictMode) ?? DuplicationBehavior::getDefaultDuplicationBehaviour();
+        }
+
         return $targetFolder->getStorage()->copyFolder($this, $targetFolder, $targetFolderName, $conflictMode);
     }
 
@@ -388,32 +399,47 @@ class Folder implements FolderInterface
      *
      * @param Folder $targetFolder Target folder to move to.
      * @param string $targetFolderName an optional destination fileName
-     * @param string $conflictMode a value of the \TYPO3\CMS\Core\Resource\DuplicationBehavior enumeration
+     * @param string|DuplicationBehavior $conflictMode
      * @return Folder New (copied) folder object.
+     * @todo change $conflictMode parameter type to DuplicationBehavior in TYPO3 v14.0
      */
     public function moveTo(Folder $targetFolder, $targetFolderName = null, $conflictMode = DuplicationBehavior::RENAME)
     {
+        if (!$conflictMode instanceof DuplicationBehavior) {
+            trigger_error(
+                'Using the non-native enumeration TYPO3\CMS\Core\Resource\DuplicationBehavior in Folder->moveTo()'
+                . ' will stop working in TYPO3 v14.0. Use native TYPO3\CMS\Core\Resource\Enum\DuplicationBehavior instead.',
+                E_USER_DEPRECATED
+            );
+            $conflictMode = DuplicationBehavior::tryFrom($conflictMode) ?? DuplicationBehavior::getDefaultDuplicationBehaviour();
+        }
+
         return $targetFolder->getStorage()->moveFolder($this, $targetFolder, $targetFolderName, $conflictMode);
     }
 
     /**
      * Checks if a file exists in this folder
-     *
-     * @param string $name
-     * @return bool
      */
-    public function hasFile($name)
+    public function hasFile(string $name): bool
     {
         return $this->storage->hasFileInFolder($name, $this);
     }
 
     /**
-     * Checks if a folder exists in this folder.
-     *
-     * @param string $name
-     * @return bool
+     * Fetches a file from a folder, must be a direct descendant of a folder.
      */
-    public function hasFolder($name)
+    public function getFile(string $fileName): ?FileInterface
+    {
+        if ($this->storage->hasFileInFolder($fileName, $this)) {
+            return $this->storage->getFileInFolder($fileName, $this);
+        }
+        return null;
+    }
+
+    /**
+     * Checks if a folder exists in this folder.
+     */
+    public function hasFolder(string $name): bool
     {
         return $this->storage->hasFolderInFolder($name, $this);
     }
@@ -510,8 +536,6 @@ class Folder implements FolderInterface
     /**
      * Sets the filters to use when listing files. These are only used if the filter mode is one of
      * FILTER_MODE_USE_OWN_FILTERS and FILTER_MODE_USE_OWN_AND_STORAGE_FILTERS
-     *
-     * @param array $filters
      */
     public function setFileAndFolderNameFilters(array $filters)
     {
@@ -535,30 +559,26 @@ class Folder implements FolderInterface
      *
      * The parent folder of the root folder is the root folder.
      *
-     * @return FolderInterface
+     * @throws InsufficientFolderAccessPermissionsException
      */
-    public function getParentFolder()
+    public function getParentFolder(): FolderInterface
     {
         return $this->getStorage()->getFolder($this->getStorage()->getFolderIdentifierFromFileIdentifier($this->getIdentifier()));
     }
 
     /**
      * Returns the modification time of the file as Unix timestamp
-     *
-     * @return int
      */
-    public function getModificationTime()
+    public function getModificationTime(): int
     {
-        return $this->storage->getFolderInfo($this)['mtime'];
+        return (int)$this->storage->getFolderInfo($this)['mtime'];
     }
 
     /**
      * Returns the creation time of the file as Unix timestamp
-     *
-     * @return int
      */
-    public function getCreationTime()
+    public function getCreationTime(): int
     {
-        return $this->storage->getFolderInfo($this)['ctime'];
+        return (int)$this->storage->getFolderInfo($this)['ctime'];
     }
 }
