@@ -19,9 +19,6 @@ use TYPO3\CMS\Core\Resource\FileInterface;
 use TYPO3\CMS\Core\Resource\FileReference;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 
-/**
- * Class AudioTagRenderer
- */
 class AudioTagRenderer implements FileRendererInterface
 {
     /**
@@ -29,7 +26,7 @@ class AudioTagRenderer implements FileRendererInterface
      *
      * @var array
      */
-    protected $possibleMimeTypes = ['audio/mpeg', 'audio/wav', 'audio/ogg'];
+    protected $possibleMimeTypes = ['audio/mpeg', 'audio/wav', 'audio/x-wav', 'audio/ogg'];
 
     /**
      * Returns the priority of the renderer
@@ -59,16 +56,13 @@ class AudioTagRenderer implements FileRendererInterface
     /**
      * Render for given File(Reference) HTML output
      *
-     * @param FileInterface $file
      * @param int|string $width TYPO3 known format; examples: 220, 200m or 200c
      * @param int|string $height TYPO3 known format; examples: 220, 200m or 200c
      * @param array $options controls = TRUE/FALSE (default TRUE), autoplay = TRUE/FALSE (default FALSE), loop = TRUE/FALSE (default FALSE)
-     * @param bool $usedPathsRelativeToCurrentScript See $file->getPublicUrl()
      * @return string
      */
-    public function render(FileInterface $file, $width, $height, array $options = [], $usedPathsRelativeToCurrentScript = false)
+    public function render(FileInterface $file, $width, $height, array $options = [])
     {
-
         // If autoplay isn't set manually check if $file is a FileReference take autoplay from there
         if (!isset($options['autoplay']) && $file instanceof FileReference) {
             $autoplay = $file->getProperty('autoplay');
@@ -82,7 +76,7 @@ class AudioTagRenderer implements FileRendererInterface
             $additionalAttributes[] = GeneralUtility::implodeAttributes($options['additionalAttributes'], true, true);
         }
         if (isset($options['data']) && is_array($options['data'])) {
-            array_walk($options['data'], function (&$value, $key) {
+            array_walk($options['data'], static function (string &$value, string $key): void {
                 $value = 'data-' . htmlspecialchars($key) . '="' . htmlspecialchars($value) . '"';
             });
             $additionalAttributes[] = implode(' ', $options['data']);
@@ -108,7 +102,7 @@ class AudioTagRenderer implements FileRendererInterface
         return sprintf(
             '<audio%s><source src="%s" type="%s"></audio>',
             empty($additionalAttributes) ? '' : ' ' . implode(' ', $additionalAttributes),
-            htmlspecialchars((string)$file->getPublicUrl($usedPathsRelativeToCurrentScript)),
+            htmlspecialchars((string)$file->getPublicUrl()),
             $file->getMimeType()
         );
     }
