@@ -15,11 +15,13 @@
 
 namespace TYPO3\CMS\Core\Resource\Rendering;
 
+use TYPO3\CMS\Core\Page\PageRenderer;
 use TYPO3\CMS\Core\Resource\File;
 use TYPO3\CMS\Core\Resource\FileInterface;
 use TYPO3\CMS\Core\Resource\FileReference;
 use TYPO3\CMS\Core\Resource\OnlineMedia\Helpers\OnlineMediaHelperInterface;
 use TYPO3\CMS\Core\Resource\OnlineMedia\Helpers\OnlineMediaHelperRegistry;
+use TYPO3\CMS\Core\Utility\GeneralUtility;
 
 /**
  * Vimeo renderer class
@@ -27,7 +29,7 @@ use TYPO3\CMS\Core\Resource\OnlineMedia\Helpers\OnlineMediaHelperRegistry;
 class VimeoRenderer implements FileRendererInterface
 {
     /**
-     * @var OnlineMediaHelperInterface
+     * @var OnlineMediaHelperInterface|false
      */
     protected $onlineMediaHelper;
 
@@ -59,8 +61,7 @@ class VimeoRenderer implements FileRendererInterface
     /**
      * Get online media helper
      *
-     * @param FileInterface $file
-     * @return bool|OnlineMediaHelperInterface
+     * @return false|OnlineMediaHelperInterface
      */
     protected function getOnlineMediaHelper(FileInterface $file)
     {
@@ -70,7 +71,7 @@ class VimeoRenderer implements FileRendererInterface
                 $orgFile = $orgFile->getOriginalFile();
             }
             if ($orgFile instanceof File) {
-                $this->onlineMediaHelper = OnlineMediaHelperRegistry::getInstance()->getOnlineMediaHelper($orgFile);
+                $this->onlineMediaHelper = GeneralUtility::makeInstance(OnlineMediaHelperRegistry::class)->getOnlineMediaHelper($orgFile);
             } else {
                 $this->onlineMediaHelper = false;
             }
@@ -81,14 +82,11 @@ class VimeoRenderer implements FileRendererInterface
     /**
      * Render for given File(Reference) html output
      *
-     * @param FileInterface $file
      * @param int|string $width TYPO3 known format; examples: 220, 200m or 200c
      * @param int|string $height TYPO3 known format; examples: 220, 200m or 200c
-     * @param array $options
-     * @param bool $usedPathsRelativeToCurrentScript See $file->getPublicUrl()
      * @return string
      */
-    public function render(FileInterface $file, $width, $height, array $options = [], $usedPathsRelativeToCurrentScript = false)
+    public function render(FileInterface $file, $width, $height, array $options = [])
     {
         $options = $this->collectOptions($options, $file);
         $src = $this->createVimeoUrl($options, $file);
@@ -102,8 +100,6 @@ class VimeoRenderer implements FileRendererInterface
     }
 
     /**
-     * @param array $options
-     * @param FileInterface $file
      * @return array
      */
     protected function collectOptions(array $options, FileInterface $file)
@@ -127,36 +123,44 @@ class VimeoRenderer implements FileRendererInterface
     }
 
     /**
-     * @param array $options
-     * @param FileInterface $file
      * @return string
      */
     protected function createVimeoUrl(array $options, FileInterface $file)
     {
-        $videoId = $this->getVideoIdFromFile($file);
+        $videoIdRaw = $this->getVideoIdFromFile($file);
+        $videoIdRaw = GeneralUtility::trimExplode('/', $videoIdRaw, true);
+
+        $videoId = $videoIdRaw[0];
+        $hash = $videoIdRaw[1] ?? null;
+
         $urlParams = [];
+        if (!empty($hash)) {
+            $urlParams[] = 'h=' . $hash;
+        }
         if (!empty($options['autoplay'])) {
             $urlParams[] = 'autoplay=1';
+            // If autoplay is enabled, enforce muted=1, see https://developer.chrome.com/blog/autoplay/
+            $urlParams[] = 'muted=1';
         }
         if (!empty($options['loop'])) {
             $urlParams[] = 'loop=1';
         }
-
+        if (!empty($options['background'])) {
+            $urlParams[] = 'background=1';
+        }
         if (isset($options['api']) && (int)$options['api'] === 1) {
             $urlParams[] = 'api=1';
         }
-        if (!empty($options['no-cookie'])) {
+        if (!isset($options['no-cookie']) || !empty($options['no-cookie'])) {
             $urlParams[] = 'dnt=1';
         }
         $urlParams[] = 'title=' . (int)!empty($options['showinfo']);
         $urlParams[] = 'byline=' . (int)!empty($options['showinfo']);
         $urlParams[] = 'portrait=0';
-
         return sprintf('https://player.vimeo.com/video/%s?%s', $videoId, implode('&', $urlParams));
     }
 
     /**
-     * @param FileInterface $file
      * @return string
      */
     protected function getVideoIdFromFile(FileInterface $file)
@@ -173,7 +177,6 @@ class VimeoRenderer implements FileRendererInterface
     /**
      * @param int|string $width
      * @param int|string $height
-     * @param array $options
      * @return array pairs of key/value; not yet html-escaped
      */
     protected function collectIframeAttributes($width, $height, array $options)
@@ -187,7 +190,7 @@ class VimeoRenderer implements FileRendererInterface
         if (isset($options['data']) && is_array($options['data'])) {
             array_walk(
                 $options['data'],
-                function (&$value, $key) use (&$attributes) {
+                static function (string $value, string|int $key) use (&$attributes): void {
                     $attributes['data-' . $key] = $value;
                 }
             );
@@ -198,7 +201,7 @@ class VimeoRenderer implements FileRendererInterface
         if ((int)$height > 0) {
             $attributes['height'] = (int)$height;
         }
-        if (isset($GLOBALS['TSFE']) && is_object($GLOBALS['TSFE']) && (isset($GLOBALS['TSFE']->config['config']['doctype']) && $GLOBALS['TSFE']->config['config']['doctype'] !== 'html5')) {
+        if ($this->shouldIncludeFrameBorderAttribute()) {
             $attributes['frameborder'] = 0;
         }
         foreach (['class', 'dir', 'id', 'lang', 'style', 'title', 'accesskey', 'tabindex', 'onclick', 'allow'] as $key) {
@@ -211,8 +214,6 @@ class VimeoRenderer implements FileRendererInterface
 
     /**
      * @internal
-     * @param array $attributes
-     * @return string
      */
     protected function implodeAttributes(array $attributes): string
     {
@@ -227,4 +228,12 @@ class VimeoRenderer implements FileRendererInterface
         }
         return implode(' ', $attributeList);
     }
+
+    /**
+     * HTML5 deprecated the "frameborder" attribute as everything should be done via styling.
+     */
+    protected function shouldIncludeFrameBorderAttribute(): bool
+    {
+        return GeneralUtility::makeInstance(PageRenderer::class)->getDocType()->shouldIncludeFrameBorderAttribute();
+    }
 }
