@@ -1,5 +1,7 @@
 <?php
 
+declare(strict_types=1);
+
 /*
  * This file is part of the TYPO3 CMS project.
  *
@@ -18,31 +20,30 @@ namespace TYPO3\CMS\Extbase\Validation\Validator;
 /**
  * Validator for general numbers
  */
-class NumberRangeValidator extends AbstractValidator
+final class NumberRangeValidator extends AbstractValidator
 {
+    protected string $notValidMessage = 'LLL:EXT:extbase/Resources/Private/Language/locallang.xlf:validator.numberrange.notvalid';
+    protected string $notInRangeMessage = 'LLL:EXT:extbase/Resources/Private/Language/locallang.xlf:validator.numberrange.range';
+
+    protected array $translationOptions = ['notValidMessage', 'notInRangeMessage'];
+
     /**
      * @var array
      */
     protected $supportedOptions = [
         'minimum' => [0, 'The minimum value to accept', 'integer'],
-        'maximum' => [PHP_INT_MAX, 'The maximum value to accept', 'integer']
+        'maximum' => [PHP_INT_MAX, 'The maximum value to accept', 'integer'],
+        'notValidMessage' => [null, 'Translation key or message for non valid value', 'string'],
+        'notInRangeMessage' => [null, 'Translation key or message for value not in range', 'string'],
     ];
 
     /**
      * The given value is valid if it is a number in the specified range.
-     *
-     * @param mixed $value The value that should be validated
      */
-    public function isValid($value)
+    public function isValid(mixed $value): void
     {
         if (!is_numeric($value)) {
-            $this->addError(
-                $this->translateErrorMessage(
-                    'validator.numberrange.notvalid',
-                    'extbase'
-                ) ?? '',
-                1221563685
-            );
+            $this->addError($this->translateErrorMessage($this->notValidMessage), 1221563685);
             return;
         }
 
@@ -56,13 +57,13 @@ class NumberRangeValidator extends AbstractValidator
         }
         if ($value < $minimum || $value > $maximum) {
             $this->addError($this->translateErrorMessage(
-                'validator.numberrange.range',
-                'extbase',
+                $this->notInRangeMessage,
+                '',
                 [
                     $minimum,
-                    $maximum
+                    $maximum,
                 ]
-            ) ?? '', 1221561046, [$minimum, $maximum]);
+            ), 1221561046, [$minimum, $maximum]);
         }
     }
 }
