@@ -1,5 +1,7 @@
 <?php
 
+declare(strict_types=1);
+
 /*
  * This file is part of the TYPO3 CMS project.
  *
@@ -16,86 +18,31 @@
 namespace TYPO3\CMS\Fluid\ViewHelpers\Form;
 
 /**
- * ViewHelper which creates a simple radio button :html:`<input type="radio">`.
+ * ViewHelper which renders a simple radio button `<input type="radio">`.
  *
- * Examples
- * ========
+ * ```
+ *   <f:form.radio name="myRadioButton" value="someValue" />
+ *   <f:form.radio property="newsletter" value="1" />
+ * ```
  *
- * Simple
- * ------
- *
- * ::
- *
- *    <f:form.radio name="myRadioButton" value="someValue" />
- *
- * Output::
- *
- *    <input type="radio" name="myRadioButton" value="someValue" />
- *
- * Preselect
- * ---------
- *
- * ::
- *
- *    <f:form.radio name="myRadioButton" value="someValue" checked="{object.value} == 5" />
- *
- * Output::
- *
- *    <input type="radio" name="myRadioButton" value="someValue" checked="checked" />
- *
- * Depending on bound ``object`` to surrounding :ref:`f:form <typo3-fluid-form>`.
- *
- * Bind to object property
- * -----------------------
- *
- * ::
- *
- *    <f:form.radio property="newsletter" value="1" /> yes
- *    <f:form.radio property="newsletter" value="0" /> no
- *
- * Output::
- *
- *    <input type="radio" name="user[newsletter]" value="1" checked="checked" /> yes
- *    <input type="radio" name="user[newsletter]" value="0" /> no
- *
- * Depending on property ``newsletter``.
+ * @see https://docs.typo3.org/permalink/t3viewhelper:typo3-fluid-form-radio
  */
-class RadioViewHelper extends AbstractFormFieldViewHelper
+final class RadioViewHelper extends AbstractFormFieldViewHelper
 {
     /**
      * @var string
      */
     protected $tagName = 'input';
 
-    /**
-     * Initialize the arguments.
-     */
-    public function initializeArguments()
+    public function initializeArguments(): void
     {
         parent::initializeArguments();
-        $this->registerArgument(
-            'errorClass',
-            'string',
-            'CSS class to set if there are errors for this ViewHelper',
-            false,
-            'f3-form-error'
-        );
+        $this->registerArgument('errorClass', 'string', 'CSS class to set if there are errors for this ViewHelper', false, 'f3-form-error');
         $this->registerArgument('checked', 'bool', 'Specifies that the input element should be preselected');
-        $this->overrideArgument('value', 'string', 'Value of input tag. Required for radio buttons', true);
-        $this->registerUniversalTagAttributes();
-        $this->registerTagAttribute(
-            'disabled',
-            'string',
-            'Specifies that the input element should be disabled when the page loads'
-        );
+        $this->registerArgument('value', 'string', 'Value of input tag. Required for radio buttons', true);
     }
 
-    /**
-     * Renders the checkbox.
-     *
-     * @return string
-     */
-    public function render()
+    public function render(): string
     {
         $checked = $this->arguments['checked'];
 
