@@ -34,12 +34,11 @@ use TYPO3\CMS\Core\Utility\Exception\MissingArrayPathException;
  */
 class FormState
 {
-
     /**
      * Constant which means that we are currently not on any page; i.e. the form
      * has never rendered before.
      */
-    const NOPAGE = -1;
+    public const NOPAGE = -1;
 
     /**
      * The last displayed page index
@@ -61,32 +60,22 @@ class FormState
         return $this->lastDisplayedPageIndex !== self::NOPAGE;
     }
 
-    /**
-     * @return int
-     */
     public function getLastDisplayedPageIndex(): int
     {
         return $this->lastDisplayedPageIndex;
     }
 
-    /**
-     * @param int $lastDisplayedPageIndex
-     */
     public function setLastDisplayedPageIndex(int $lastDisplayedPageIndex)
     {
         $this->lastDisplayedPageIndex = $lastDisplayedPageIndex;
     }
 
-    /**
-     * @return array
-     */
     public function getFormValues(): array
     {
         return $this->formValues;
     }
 
     /**
-     * @param string $propertyPath
      * @param mixed $value
      */
     public function setFormValue(string $propertyPath, $value)
@@ -100,7 +89,6 @@ class FormState
     }
 
     /**
-     * @param string $propertyPath
      * @return mixed
      */
     public function getFormValue(string $propertyPath)
