@@ -23,20 +23,16 @@ use TYPO3\CMS\Form\Slot\FilePersistenceSlot;
  */
 class ImportExportHook
 {
-
-    /**
-     * @param array $params
-     */
     public function beforeAddSysFileRecordOnImport(array $params): void
     {
         $fileRecord = $params['fileRecord'];
         $temporaryFile = $params['temporaryFile'];
 
-        $formPersistenceSlot = GeneralUtility::makeInstance(FilePersistenceSlot::class);
-        $formPersistenceSlot->allowInvocation(
+        $filePersistenceSlot = GeneralUtility::makeInstance(FilePersistenceSlot::class);
+        $filePersistenceSlot->allowInvocation(
             FilePersistenceSlot::COMMAND_FILE_ADD,
             implode(':', [$fileRecord['storage'], $fileRecord['identifier']]),
-            $formPersistenceSlot->getContentSignature(file_get_contents($temporaryFile))
+            $filePersistenceSlot->getContentSignature(file_get_contents($temporaryFile))
         );
     }
 }
