@@ -17,55 +17,57 @@ declare(strict_types=1);
 
 namespace TYPO3\CMS\Form\Mvc\Property\TypeConverter;
 
+use TYPO3\CMS\Core\Crypto\HashService;
 use TYPO3\CMS\Core\Crypto\Random;
+use TYPO3\CMS\Core\Http\UploadedFile;
 use TYPO3\CMS\Core\Log\LogManager;
 use TYPO3\CMS\Core\Resource\Exception\FolderDoesNotExistException;
-use TYPO3\CMS\Core\Resource\File as File;
+use TYPO3\CMS\Core\Resource\File;
 use TYPO3\CMS\Core\Resource\FileReference as CoreFileReference;
 use TYPO3\CMS\Core\Resource\Folder;
 use TYPO3\CMS\Core\Resource\ResourceFactory;
+use TYPO3\CMS\Core\Resource\ResourceInstructionTrait;
 use TYPO3\CMS\Core\Resource\Security\FileNameValidator;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\StringUtility;
-use TYPO3\CMS\Extbase\Domain\Model\AbstractFileFolder;
+use TYPO3\CMS\Extbase\Domain\Model\FileReference;
 use TYPO3\CMS\Extbase\Error\Error;
 use TYPO3\CMS\Extbase\Persistence\PersistenceManagerInterface;
 use TYPO3\CMS\Extbase\Property\PropertyMappingConfigurationInterface;
 use TYPO3\CMS\Extbase\Property\TypeConverter\AbstractTypeConverter;
-use TYPO3\CMS\Extbase\Security\Cryptography\HashService;
 use TYPO3\CMS\Extbase\Validation\Validator\AbstractValidator;
 use TYPO3\CMS\Form\Mvc\Property\Exception\TypeConverterException;
+use TYPO3\CMS\Form\Security\HashScope;
 use TYPO3\CMS\Form\Service\TranslationService;
 use TYPO3\CMS\Form\Slot\ResourcePublicationSlot;
 
 /**
- * Class UploadedFileReferenceConverter
- *
  * Scope: frontend
  * @internal
  */
 class UploadedFileReferenceConverter extends AbstractTypeConverter
 {
+    use ResourceInstructionTrait;
 
     /**
      * Folder where the file upload should go to (including storage).
      */
-    const CONFIGURATION_UPLOAD_FOLDER = 1;
+    public const CONFIGURATION_UPLOAD_FOLDER = 1;
 
     /**
      * How to handle a upload when the name of the uploaded file conflicts.
      */
-    const CONFIGURATION_UPLOAD_CONFLICT_MODE = 2;
+    public const CONFIGURATION_UPLOAD_CONFLICT_MODE = 2;
 
     /**
      * Random seed to be used for deriving storage sub-folders.
      */
-    const CONFIGURATION_UPLOAD_SEED = 3;
+    public const CONFIGURATION_UPLOAD_SEED = 3;
 
     /**
      * Validator for file types
      */
-    const CONFIGURATION_FILE_VALIDATORS = 4;
+    public const CONFIGURATION_FILE_VALIDATORS = 4;
 
     /**
      * @var string
@@ -79,23 +81,6 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
      */
     protected $defaultConflictMode = 'rename';
 
-    /**
-     * @var array
-     */
-    protected $sourceTypes = ['array'];
-
-    /**
-     * @var string
-     */
-    protected $targetType = PseudoFileReference::class;
-
-    /**
-     * Take precedence over the available FileReferenceConverter
-     *
-     * @var int
-     */
-    protected $priority = 12;
-
     /**
      * @var PseudoFileReference[]
      */
@@ -107,7 +92,7 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
     protected $resourceFactory;
 
     /**
-     * @var \TYPO3\CMS\Extbase\Security\Cryptography\HashService
+     * @var HashService
      */
     protected $hashService;
 
@@ -117,7 +102,6 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
     protected $persistenceManager;
 
     /**
-     * @param \TYPO3\CMS\Core\Resource\ResourceFactory $resourceFactory
      * @internal
      */
     public function injectResourceFactory(ResourceFactory $resourceFactory)
@@ -126,7 +110,6 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
     }
 
     /**
-     * @param \TYPO3\CMS\Extbase\Security\Cryptography\HashService $hashService
      * @internal
      */
     public function injectHashService(HashService $hashService)
@@ -135,7 +118,6 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
     }
 
     /**
-     * @param \TYPO3\CMS\Extbase\Persistence\PersistenceManagerInterface $persistenceManager
      * @internal
      */
     public function injectPersistenceManager(PersistenceManagerInterface $persistenceManager)
@@ -147,15 +129,16 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
      * Actually convert from $source to $targetType, taking into account the fully
      * built $convertedChildProperties and $configuration.
      *
-     * @param array $source
+     * @param array|UploadedFile $source
      * @param string $targetType
-     * @param array $convertedChildProperties
-     * @param PropertyMappingConfigurationInterface $configuration
-     * @return AbstractFileFolder|Error|null
+     * @return File|FileReference|Folder|Error|null
      * @internal
      */
-    public function convertFrom($source, $targetType, array $convertedChildProperties = [], PropertyMappingConfigurationInterface $configuration = null)
+    public function convertFrom($source, $targetType, array $convertedChildProperties = [], ?PropertyMappingConfigurationInterface $configuration = null)
     {
+        if ($source instanceof UploadedFile) {
+            $source = $this->convertUploadedFileToUploadInfoArray($source);
+        }
         // slot/listener using `FileDumpController` instead of direct public URL in (later) rendering process
         $resourcePublicationSlot = GeneralUtility::makeInstance(ResourcePublicationSlot::class);
         if (!isset($source['error']) || $source['error'] === \UPLOAD_ERR_NO_FILE) {
@@ -163,13 +146,13 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
                 try {
                     // File references use numeric resource pointers, direct
                     // file relations are using "file:" prefix (e.g. "file:5")
-                    $resourcePointer = $this->hashService->validateAndStripHmac($source['submittedFile']['resourcePointer']);
-                    if (strpos($resourcePointer, 'file:') === 0) {
+                    $resourcePointer = $this->hashService->validateAndStripHmac($source['submittedFile']['resourcePointer'], HashScope::ResourcePointer->prefix());
+                    if (str_starts_with($resourcePointer, 'file:')) {
                         $fileUid = (int)substr($resourcePointer, 5);
                         $resource = $this->createFileReferenceFromFalFileObject($this->resourceFactory->getFileObject($fileUid));
                     } else {
                         $resource = $this->createFileReferenceFromFalFileReferenceObject(
-                            $this->resourceFactory->getFileReferenceObject($resourcePointer),
+                            $this->resourceFactory->getFileReferenceObject((int)$resourcePointer),
                             (int)$resourcePointer
                         );
                     }
@@ -183,7 +166,7 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
         }
 
         if ($source['error'] !== \UPLOAD_ERR_OK) {
-            return $this->objectManager->get(Error::class, $this->getUploadErrorMessage($source['error']), 1471715915);
+            return GeneralUtility::makeInstance(Error::class, $this->getUploadErrorMessage($source['error']), 1471715915);
         }
 
         if (isset($this->convertedResources[$source['tmp_name']])) {
@@ -200,7 +183,7 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
         } catch (TypeConverterException $e) {
             return $e->getError();
         } catch (\Exception $e) {
-            return $this->objectManager->get(Error::class, $e->getMessage(), $e->getCode());
+            return GeneralUtility::makeInstance(Error::class, $e->getMessage(), $e->getCode());
         }
 
         $this->convertedResources[$source['tmp_name']] = $resource;
@@ -209,10 +192,6 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
 
     /**
      * Import a resource and respect configuration given for properties
-     *
-     * @param array $uploadInfo
-     * @param PropertyMappingConfigurationInterface $configuration
-     * @return PseudoFileReference
      */
     protected function importUploadedResource(
         array $uploadInfo,
@@ -243,13 +222,16 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
 
         $uploadFolder = $this->provideUploadFolder($uploadFolderId);
         // current folder name, derived from public random seed (`formSession`)
-        $currentName = 'form_' . GeneralUtility::hmac($seed, self::class);
-        $uploadFolder = $this->provideTargetFolder($uploadFolder, $currentName);
+        $currentName = 'form_' . $this->hashService->hmac($seed, self::class);
         // sub-folder in $uploadFolder with 160 bit of derived entropy (.../form_<40-chars-hash>/actual.file)
+        $uploadFolder = $this->provideTargetFolder($uploadFolder, $currentName);
+        // allow skipping the consistency check, since custom validators have already been executed
+        $this->skipResourceConsistencyCheckForUploads($uploadFolder->getStorage(), $uploadInfo);
+        /** @var File $uploadedFile */
         $uploadedFile = $uploadFolder->addUploadedFile($uploadInfo, $conflictMode);
 
-        $resourcePointer = isset($uploadInfo['submittedFile']['resourcePointer']) && strpos($uploadInfo['submittedFile']['resourcePointer'], 'file:') === false
-            ? (int)$this->hashService->validateAndStripHmac($uploadInfo['submittedFile']['resourcePointer'])
+        $resourcePointer = isset($uploadInfo['submittedFile']['resourcePointer']) && !str_contains($uploadInfo['submittedFile']['resourcePointer'], 'file:')
+            ? (int)$this->hashService->validateAndStripHmac($uploadInfo['submittedFile']['resourcePointer'], HashScope::ResourcePointer->prefix())
             : null;
 
         $fileReferenceModel = $this->createFileReferenceFromFalFileObject($uploadedFile, $resourcePointer);
@@ -257,14 +239,9 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
         return $fileReferenceModel;
     }
 
-    /**
-     * @param File $file
-     * @param int $resourcePointer
-     * @return PseudoFileReference
-     */
     protected function createFileReferenceFromFalFileObject(
         File $file,
-        int $resourcePointer = null
+        ?int $resourcePointer = null
     ): PseudoFileReference {
         $fileReference = $this->resourceFactory->createFileReferenceObject(
             [
@@ -281,17 +258,13 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
      * In case no $resourcePointer is given a new file reference domain object
      * will be returned. Otherwise the file reference is reconstituted from
      * storage and will be updated(!) with the provided $falFileReference.
-     *
-     * @param CoreFileReference $falFileReference
-     * @param int $resourcePointer
-     * @return PseudoFileReference
      */
     protected function createFileReferenceFromFalFileReferenceObject(
         CoreFileReference $falFileReference,
-        int $resourcePointer = null
+        ?int $resourcePointer = null
     ): PseudoFileReference {
         if ($resourcePointer === null) {
-            $fileReference = $this->objectManager->get(PseudoFileReference::class);
+            $fileReference = GeneralUtility::makeInstance(PseudoFileReference::class);
         } else {
             $fileReference = $this->persistenceManager->getObjectByIdentifier($resourcePointer, PseudoFileReference::class, false);
         }
@@ -303,9 +276,6 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
     /**
      * Returns a human-readable message for the given PHP file upload error
      * constant.
-     *
-     * @param int $errorCode
-     * @return string
      */
     protected function getUploadErrorMessage(int $errorCode): string
     {
@@ -313,36 +283,33 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
         switch ($errorCode) {
             case \UPLOAD_ERR_INI_SIZE:
                 $logger->error('The uploaded file exceeds the upload_max_filesize directive in php.ini.', []);
-                return TranslationService::getInstance()->translate('upload.error.150530345', null, 'EXT:form/Resources/Private/Language/locallang.xlf');
+                return GeneralUtility::makeInstance(TranslationService::class)->translate('upload.error.150530345', null, 'EXT:form/Resources/Private/Language/locallang.xlf');
             case \UPLOAD_ERR_FORM_SIZE:
                 $logger->error('The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form.', []);
-                return TranslationService::getInstance()->translate('upload.error.150530345', null, 'EXT:form/Resources/Private/Language/locallang.xlf');
+                return GeneralUtility::makeInstance(TranslationService::class)->translate('upload.error.150530345', null, 'EXT:form/Resources/Private/Language/locallang.xlf');
             case \UPLOAD_ERR_PARTIAL:
                 $logger->error('The uploaded file was only partially uploaded.', []);
-                return TranslationService::getInstance()->translate('upload.error.150530346', null, 'EXT:form/Resources/Private/Language/locallang.xlf');
+                return GeneralUtility::makeInstance(TranslationService::class)->translate('upload.error.150530346', null, 'EXT:form/Resources/Private/Language/locallang.xlf');
             case \UPLOAD_ERR_NO_FILE:
                 $logger->error('No file was uploaded.', []);
-                return TranslationService::getInstance()->translate('upload.error.150530347', null, 'EXT:form/Resources/Private/Language/locallang.xlf');
+                return GeneralUtility::makeInstance(TranslationService::class)->translate('upload.error.150530347', null, 'EXT:form/Resources/Private/Language/locallang.xlf');
             case \UPLOAD_ERR_NO_TMP_DIR:
                 $logger->error('Missing a temporary folder.', []);
-                return TranslationService::getInstance()->translate('upload.error.150530348', null, 'EXT:form/Resources/Private/Language/locallang.xlf');
+                return GeneralUtility::makeInstance(TranslationService::class)->translate('upload.error.150530348', null, 'EXT:form/Resources/Private/Language/locallang.xlf');
             case \UPLOAD_ERR_CANT_WRITE:
                 $logger->error('Failed to write file to disk.', []);
-                return TranslationService::getInstance()->translate('upload.error.150530348', null, 'EXT:form/Resources/Private/Language/locallang.xlf');
+                return GeneralUtility::makeInstance(TranslationService::class)->translate('upload.error.150530348', null, 'EXT:form/Resources/Private/Language/locallang.xlf');
             case \UPLOAD_ERR_EXTENSION:
                 $logger->error('File upload stopped by extension.', []);
-                return TranslationService::getInstance()->translate('upload.error.150530348', null, 'EXT:form/Resources/Private/Language/locallang.xlf');
+                return GeneralUtility::makeInstance(TranslationService::class)->translate('upload.error.150530348', null, 'EXT:form/Resources/Private/Language/locallang.xlf');
             default:
                 $logger->error('Unknown upload error.', []);
-                return TranslationService::getInstance()->translate('upload.error.150530348', null, 'EXT:form/Resources/Private/Language/locallang.xlf');
+                return GeneralUtility::makeInstance(TranslationService::class)->translate('upload.error.150530348', null, 'EXT:form/Resources/Private/Language/locallang.xlf');
         }
     }
 
     /**
      * Ensures that upload folder exists, creates it if it does not.
-     *
-     * @param string $uploadFolderIdentifier
-     * @return Folder
      */
     protected function provideUploadFolder(string $uploadFolderIdentifier): Folder
     {
@@ -360,10 +327,6 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
 
     /**
      * Ensures that particular target folder exists, creates it if it does not.
-     *
-     * @param Folder $parentFolder
-     * @param string $folderName
-     * @return Folder
      */
     protected function provideTargetFolder(Folder $parentFolder, string $folderName): Folder
     {
@@ -375,8 +338,6 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
     /**
      * Creates empty index.html file to avoid directory indexing,
      * in case it does not exist yet.
-     *
-     * @param Folder $parentFolder
      */
     protected function provideFolderInitialization(Folder $parentFolder): void
     {
@@ -384,4 +345,15 @@ class UploadedFileReferenceConverter extends AbstractTypeConverter
             $parentFolder->createFile('index.html');
         }
     }
+
+    protected function convertUploadedFileToUploadInfoArray(UploadedFile $uploadedFile): array
+    {
+        return [
+            'name' => $uploadedFile->getClientFilename(),
+            'tmp_name' => $uploadedFile->getTemporaryFileName(),
+            'size' => $uploadedFile->getSize(),
+            'error' => $uploadedFile->getError(),
+            'type' => $uploadedFile->getClientMediaType(),
+        ];
+    }
 }
