@@ -19,6 +19,4 @@ namespace TYPO3\CMS\Form\Mvc\Validation\Exception;
 
 use TYPO3\CMS\Form\Exception;
 
-class InvalidValidationOptionsException extends Exception
-{
-}
+class InvalidValidationOptionsException extends Exception {}
