@@ -27,10 +27,13 @@ interface ExceptionHandlerInterface
      * The handler can decide whether to re-throw the exception or
      * return a nice error message for production context.
      *
-     * @param \Exception $exception
-     * @param AbstractContentObject $contentObject
      * @param array $contentObjectConfiguration
      * @return string
      */
-    public function handle(\Exception $exception, AbstractContentObject $contentObject = null, $contentObjectConfiguration = []);
+    public function handle(\Exception $exception, ?AbstractContentObject $contentObject = null, $contentObjectConfiguration = []);
+
+    /**
+     * Used to pass the TypoScript configuration to the exception handler
+     */
+    public function setConfiguration(array $configuration): void;
 }
