@@ -20,48 +20,29 @@ namespace TYPO3\CMS\Frontend\Http;
 use Psr\Http\Message\ResponseInterface;
 use Psr\Http\Message\ServerRequestInterface;
 use Psr\Http\Server\RequestHandlerInterface;
-use TYPO3\CMS\Core\Configuration\ConfigurationManager;
 use TYPO3\CMS\Core\Context\Context;
 use TYPO3\CMS\Core\Context\DateTimeAspect;
 use TYPO3\CMS\Core\Context\UserAspect;
 use TYPO3\CMS\Core\Context\VisibilityAspect;
 use TYPO3\CMS\Core\Context\WorkspaceAspect;
-use TYPO3\CMS\Core\Core\Environment;
 use TYPO3\CMS\Core\Core\SystemEnvironmentBuilder;
+use TYPO3\CMS\Core\Domain\DateTimeFactory;
 use TYPO3\CMS\Core\Http\AbstractApplication;
-use TYPO3\CMS\Core\Http\RedirectResponse;
 
 /**
  * Entry point for the TYPO3 Frontend
  */
 class Application extends AbstractApplication
 {
-    /**
-     * @var ConfigurationManager
-     */
-    protected $configurationManager;
-
-    /**
-     * @var Context
-     */
-    protected $context;
-
     public function __construct(
         RequestHandlerInterface $requestHandler,
-        ConfigurationManager $configurationManager,
-        Context $context
+        protected readonly Context $context,
     ) {
         $this->requestHandler = $requestHandler;
-        $this->configurationManager = $configurationManager;
-        $this->context = $context;
     }
 
-    protected function handle(ServerRequestInterface $request): ResponseInterface
+    public function handle(ServerRequestInterface $request): ResponseInterface
     {
-        if (!$this->checkIfEssentialConfigurationExists()) {
-            return $this->installToolRedirect();
-        }
-
         // Create new request object having applicationType "I am a frontend request" attribute.
         $request = $request->withAttribute('applicationType', SystemEnvironmentBuilder::REQUESTTYPE_FE);
 
@@ -69,34 +50,12 @@ class Application extends AbstractApplication
         return parent::handle($request);
     }
 
-    /**
-     * Check if LocalConfiguration.php and PackageStates.php exist
-     *
-     * @return bool TRUE when the essential configuration is available, otherwise FALSE
-     */
-    protected function checkIfEssentialConfigurationExists(): bool
-    {
-        return file_exists($this->configurationManager->getLocalConfigurationFileLocation())
-            && file_exists(Environment::getLegacyConfigPath() . '/PackageStates.php');
-    }
-
-    /**
-     * Create a PSR-7 Response that redirects to the install tool
-     *
-     * @return ResponseInterface
-     */
-    protected function installToolRedirect(): ResponseInterface
-    {
-        $path = TYPO3_mainDir . 'install.php';
-        return new RedirectResponse($path, 302);
-    }
-
     /**
      * Initializes the Context used for accessing data and finding out the current state of the application
      */
     protected function initializeContext(): void
     {
-        $this->context->setAspect('date', new DateTimeAspect(new \DateTimeImmutable('@' . $GLOBALS['EXEC_TIME'])));
+        $this->context->setAspect('date', new DateTimeAspect(DateTimeFactory::createFromTimestamp($GLOBALS['EXEC_TIME'])));
         $this->context->setAspect('visibility', new VisibilityAspect());
         $this->context->setAspect('workspace', new WorkspaceAspect(0));
         $this->context->setAspect('backend.user', new UserAspect(null));
