@@ -20,34 +20,24 @@ namespace TYPO3\CMS\Install\SystemEnvironment;
 use TYPO3\CMS\Core\Information\Typo3Information;
 use TYPO3\CMS\Core\Messaging\FlashMessage;
 use TYPO3\CMS\Core\Messaging\FlashMessageQueue;
+use TYPO3\CMS\Core\Type\ContextualFeedbackSeverity;
+use TYPO3\CMS\Core\Utility\GeneralUtility;
 
 /**
- * Check system environment status
+ * Check system environment status.
  *
  * This class is a hardcoded requirement check of the underlying
  * server and PHP system.
  *
  * The class *must not* check for any TYPO3 specific things like
- * specific configuration values or directories. It should not fail
- * if there is no TYPO3 at all.
+ * specific configuration values or directories.
  *
- * The only core code used is the class loader
+ * This class is instantiated as a very early during installation.
  *
- * This class is instantiated as the *very first* class during
- * installation. It is meant to be *standalone* und must not have
- * any requirements, except the status classes. It must be possible
- * to run this script separated from the rest of the core, without
- * dependencies.
- *
- * This means especially:
+ * Be picky with dependencies here:
  * * No hooks or anything like that
- * * No usage of *any* TYPO3 code like GeneralUtility
- * * No require of anything but the status classes
  * * No localization
- *
- * The status messages and title *must not* include HTML, use plain
- * text only. The return values of this class are not bound to HTML
- * and can be used in different scopes (eg. as json array).
+ * * Only low level ext:core classes if free of side effects
  *
  * @internal This class is only meant to be used within EXT:install and is not part of the TYPO3 Core API.
  */
@@ -64,13 +54,15 @@ class Check implements CheckInterface
     protected $requiredPhpExtensions = [
         'filter',
         'gd',
-        'hash',
+        'intl',
         'json',
         'libxml',
+        'mbstring',
         'PDO',
         'session',
         'SPL',
         'standard',
+        'tokenizer',
         'xml',
         'zip',
         'zlib',
@@ -80,9 +72,9 @@ class Check implements CheckInterface
      * @var string[]
      */
     protected $suggestedPhpExtensions = [
+        'exif' => 'This extension is used to detect the orientation of uploaded images.',
         'fileinfo' => 'This extension is used for proper file type detection in the File Abstraction Layer.',
-        'intl' => 'This extension is used for correct language and locale handling.',
-        'openssl' => 'This extension is used for sending SMTP mails over an encrypted channel endpoint, and for extensions such as "rsaauth".'
+        'openssl' => 'This extension is used for sending SMTP mails over an encrypted channel endpoint.',
     ];
 
     public function __construct()
@@ -97,8 +89,6 @@ class Check implements CheckInterface
 
     /**
      * Get all status information as array with status objects
-     *
-     * @return FlashMessageQueue
      */
     public function getStatus(): FlashMessageQueue
     {
@@ -142,7 +132,7 @@ class Check implements CheckInterface
     {
         $includePath = (string)ini_get('include_path');
         $delimiter = $this->isWindowsOs() ? ';' : ':';
-        $pathArray = $this->trimExplode($delimiter, $includePath);
+        $pathArray = GeneralUtility::trimExplode($delimiter, $includePath, true);
         if (!in_array('.', $pathArray)) {
             $this->messageQueue->enqueue(new FlashMessage(
                 'include_path = ' . implode(' ', $pathArray) . LF
@@ -151,7 +141,7 @@ class Check implements CheckInterface
                     . ' it is an unusual setting that may introduce problems for'
                     . ' some extensions.',
                 'Current directory (./) is not within PHP include path',
-                FlashMessage::WARNING
+                ContextualFeedbackSeverity::WARNING
             ));
         } else {
             $this->messageQueue->enqueue(new FlashMessage(
@@ -175,7 +165,7 @@ class Check implements CheckInterface
                     . ' the server - ALL content in the forms are discarded and therefore'
                     . ' nothing at all will be editable if you don\'t set this flag!',
                 'File uploads not allowed in PHP',
-                FlashMessage::ERROR
+                ContextualFeedbackSeverity::ERROR
             ));
         } else {
             $this->messageQueue->enqueue(new FlashMessage(
@@ -200,7 +190,7 @@ class Check implements CheckInterface
                     . ' exceeds the allowed size for POST requests. Therefore the'
                     . ' file uploads can also not be larger than ' . ini_get('post_max_size') . '.',
                 'Maximum size for POST requests is smaller than maximum upload filesize in PHP',
-                FlashMessage::ERROR
+                ContextualFeedbackSeverity::ERROR
             ));
         } elseif ($maximumPostSize === $maximumUploadFilesize) {
             $this->messageQueue->enqueue(new FlashMessage(
@@ -230,7 +220,7 @@ class Check implements CheckInterface
                     . ' To be safe, set a limit in PHP, but with a minimum of ' . $recommendedMemoryLimit . 'MB:' . LF
                     . 'memory_limit=' . $recommendedMemoryLimit . 'M',
                 'Unlimited memory limit for PHP',
-                FlashMessage::WARNING
+                ContextualFeedbackSeverity::WARNING
             ));
         } elseif ($memoryLimit < 1024 * 1024 * $minimumMemoryLimit) {
             $this->messageQueue->enqueue(new FlashMessage(
@@ -240,7 +230,7 @@ class Check implements CheckInterface
                     . ' We recommend a minimum PHP memory limit of ' . $recommendedMemoryLimit . 'MB:' . LF
                     . 'memory_limit=' . $recommendedMemoryLimit . 'M',
                 'PHP Memory limit below ' . $minimumMemoryLimit . 'MB',
-                FlashMessage::ERROR
+                ContextualFeedbackSeverity::ERROR
             ));
         } elseif ($memoryLimit < 1024 * 1024 * $recommendedMemoryLimit) {
             $this->messageQueue->enqueue(new FlashMessage(
@@ -253,7 +243,7 @@ class Check implements CheckInterface
                     . $recommendedMemoryLimit . 'MB:' . LF
                     . 'memory_limit=' . $recommendedMemoryLimit . 'M',
                 'PHP Memory limit below ' . $recommendedMemoryLimit . 'MB',
-                FlashMessage::WARNING
+                ContextualFeedbackSeverity::WARNING
             ));
         } else {
             $this->messageQueue->enqueue(new FlashMessage(
@@ -268,14 +258,14 @@ class Check implements CheckInterface
      */
     protected function checkPhpVersion()
     {
-        $minimumPhpVersion = '7.2.0';
+        $minimumPhpVersion = '8.1.0';
         $currentPhpVersion = PHP_VERSION;
         if (version_compare($currentPhpVersion, $minimumPhpVersion) < 0) {
             $this->messageQueue->enqueue(new FlashMessage(
                 'Your PHP version ' . $currentPhpVersion . ' is too old. TYPO3 CMS does not run'
                     . ' with this version. Update to at least PHP ' . $minimumPhpVersion,
                 'PHP version too low',
-                FlashMessage::ERROR
+                ContextualFeedbackSeverity::ERROR
             ));
         } else {
             $this->messageQueue->enqueue(new FlashMessage(
@@ -297,7 +287,7 @@ class Check implements CheckInterface
                     . ' in your environment. Change your environment to provide this extension'
                     . ' in with minimum version ' . $minimumPcreVersion . '.',
                 'PHP extension pcre not loaded',
-                FlashMessage::ERROR
+                ContextualFeedbackSeverity::ERROR
             ));
         } else {
             $installedPcreVersionString = trim(PCRE_VERSION); // '8.39 2016-06-14'
@@ -308,7 +298,7 @@ class Check implements CheckInterface
                     'Your PCRE version ' . PCRE_VERSION . ' is too old. TYPO3 CMS may trigger PHP segmentantion'
                         . ' faults with this version. Update to at least PCRE ' . $minimumPcreVersion,
                     'PCRE version too low',
-                    FlashMessage::ERROR
+                    ContextualFeedbackSeverity::ERROR
                 ));
             } else {
                 $this->messageQueue->enqueue(new FlashMessage(
@@ -335,7 +325,7 @@ class Check implements CheckInterface
                     . ' if it is reduced to ' . $recommendedMaximumExecutionTime . ' seconds:' . LF
                     . 'max_execution_time=' . $recommendedMaximumExecutionTime,
                 'Infinite PHP script execution time',
-                FlashMessage::WARNING
+                ContextualFeedbackSeverity::WARNING
             ));
         } elseif ($currentMaximumExecutionTime < $minimumMaximumExecutionTime) {
             $this->messageQueue->enqueue(new FlashMessage(
@@ -344,7 +334,7 @@ class Check implements CheckInterface
                     . ' It is recommended to raise the limit to ' . $recommendedMaximumExecutionTime . ' seconds:' . LF
                     . 'max_execution_time=' . $recommendedMaximumExecutionTime,
                 'Low PHP script execution time',
-                FlashMessage::ERROR
+                ContextualFeedbackSeverity::ERROR
             ));
         } elseif ($currentMaximumExecutionTime < $recommendedMaximumExecutionTime) {
             $this->messageQueue->enqueue(new FlashMessage(
@@ -356,7 +346,7 @@ class Check implements CheckInterface
                     . ' and maybe raise the limit to ' . $recommendedMaximumExecutionTime . ' seconds:' . LF
                     . 'max_execution_time=' . $recommendedMaximumExecutionTime,
                 'Low PHP script execution time',
-                FlashMessage::WARNING
+                ContextualFeedbackSeverity::WARNING
             ));
         } else {
             $this->messageQueue->enqueue(new FlashMessage(
@@ -374,38 +364,80 @@ class Check implements CheckInterface
         $disabledFunctions = trim((string)ini_get('disable_functions'));
 
         // Filter "disable_functions"
-        $disabledFunctionsArray = $this->trimExplode(',', $disabledFunctions);
+        $disabledFunctionsArray = GeneralUtility::trimExplode(',', $disabledFunctions, true);
 
         // Array with strings to find
         $findStrings = [
             // Disabled by default on Ubuntu OS but this is okay since the Core does not use them
             'pcntl_',
         ];
+
+        // List of disable_functions which would not trigger an error, but only a warning
+        $configuredAllowedDisableFunctions = $GLOBALS['TYPO3_CONF_VARS']['SYS']['allowedPhpDisableFunctions'] ?? [];
+        if (!is_array($configuredAllowedDisableFunctions)) {
+            $configuredAllowedDisableFunctions = [];
+        }
+        $foundAllowedDisableFunctions = [];
+
+        // Iterate all functions that are currently disabled by PHP ($disabledFunctionsArray).
+        // Each disabled function ($disabledFunction) is checked whether it may be acceptable
+        // to be disabled (based on $configuredAllowedDisableFunctions).
+        // If good to be disabled: Remove from $disabledFunctionsArray (which is emitted later on),
+        //                         and also add to $foundAllowedDisableFunctions for reporting.
+        // Else: Check if the function is maybe whitelisted because unused by core ($findStrings)
+        //       and if so, also remove from $disabledFunctionsArray
+        // What remains then in $disabledFunctionsArray is the list of disabled functions that
+        // need to be reported.
         foreach ($disabledFunctionsArray as $key => $disabledFunction) {
+            if (in_array($disabledFunction, $configuredAllowedDisableFunctions, true)) {
+                unset($disabledFunctionsArray[$key]);
+                $foundAllowedDisableFunctions[] = $disabledFunction;
+                continue;
+            }
+
             foreach ($findStrings as $findString) {
-                if (strpos($disabledFunction, $findString) !== false) {
+                if (str_contains($disabledFunction, $findString)) {
                     unset($disabledFunctionsArray[$key]);
                 }
             }
         }
 
         if ($disabledFunctions !== '') {
-            if (!empty($disabledFunctionsArray)) {
+            // Error for disable_functions which are not explicitly allowed
+            if ($disabledFunctionsArray !== []) {
                 $this->messageQueue->enqueue(new FlashMessage(
-                    'disable_functions=' . implode(' ', explode(',', $disabledFunctions)) . LF
-                        . 'These function(s) are disabled. TYPO3 uses some of those, so there might be trouble.'
-                        . ' TYPO3 is designed to use the default set of PHP functions plus some common extensions.'
-                        . ' Possibly these functions are disabled'
-                        . ' due to security considerations and most likely the list would include a function like'
-                        . ' exec() which is used by TYPO3 at various places. Depending on which exact functions'
-                        . ' are disabled, some parts of the system may just break without further notice.',
+                    'disable_functions=' . implode(' ', $disabledFunctionsArray) . LF
+                    . '- These function(s) are disabled. TYPO3 uses some of those, so there might be trouble.'
+                    . ' TYPO3 is designed to use the default set of PHP functions plus some common extensions.'
+                    . ' Possibly these functions are disabled'
+                    . ' due to security considerations and often the list would include a function like'
+                    . ' exec() which is used by TYPO3 at various places. Depending on which exact functions'
+                    . ' are disabled, some parts of the system may just break without further notice. Known acceptable'
+                    . ' exemptions can be muted by adding those to'
+                    . ' $GLOBALS[\'TYPO3_CONF_VARS\'][\'SYS\'][\'allowedPhpDisableFunctions\'] in your system'
+                    . ' configuration.',
                     'Some PHP functions disabled',
-                    FlashMessage::ERROR
+                    ContextualFeedbackSeverity::ERROR
                 ));
-            } else {
+            }
+            // Warning for disable_functions which are explicitly allowed
+            if ($foundAllowedDisableFunctions !== []) {
+                $this->messageQueue->enqueue(new FlashMessage(
+                    'disable_functions=' . implode(' ', $foundAllowedDisableFunctions) . LF
+                    . '- These function(s) are disabled. TYPO3 or installed extensions may use some of those, but the'
+                    . ' error reporting for them is explicitly muted in your installation via configuration variable'
+                    . ' $GLOBALS[\'TYPO3_CONF_VARS\'][\'SYS\'][\'allowedPhpDisableFunctions\'].'
+                    . ' Please ensure by yourself that these functions are not used by TYPO3 or any other installed package.',
+                    'Some PHP functions disabled, but considered irrelevant',
+                    ContextualFeedbackSeverity::WARNING
+                ));
+            }
+            // Notice for known irrelevant functions (e.g. pcntl_*)
+            // Only shown if none of the two FlashMessages above are emitted.
+            if ($disabledFunctionsArray === [] && $foundAllowedDisableFunctions === []) {
                 $this->messageQueue->enqueue(new FlashMessage(
                     'disable_functions=' . implode(' ', explode(',', $disabledFunctions)) . LF
-                        . 'These function(s) are disabled. TYPO3 uses currently none of those, so you are good to go.',
+                        . '- These function(s) are disabled. TYPO3 uses currently none of those, so you are good to go.',
                     'Some PHP functions currently disabled but OK'
                 ));
             }
@@ -432,7 +464,7 @@ class Check implements CheckInterface
                     . ' TYPO3 CMS core code is linked to some different location.'
                     . ' If that is a problem, the setting must be changed.',
                 'doc_root is set',
-                FlashMessage::NOTICE
+                ContextualFeedbackSeverity::NOTICE
             ));
         } else {
             $this->messageQueue->enqueue(new FlashMessage(
@@ -456,7 +488,7 @@ class Check implements CheckInterface
                     . ' if for example your TYPO3 CMS core is linked to a different directory'
                     . ' not included in this path.',
                 'PHP open_basedir is set',
-                FlashMessage::NOTICE
+                ContextualFeedbackSeverity::NOTICE
             ));
         } else {
             $this->messageQueue->enqueue(new FlashMessage(
@@ -485,7 +517,7 @@ class Check implements CheckInterface
                         . ' if the value is not raised significantly to:' . LF
                         . 'xdebug.max_nesting_level=' . $recommendedMaxNestingLevel,
                     'PHP xdebug.max_nesting_level is critically low',
-                    FlashMessage::ERROR
+                    ContextualFeedbackSeverity::ERROR
                 ));
             } elseif ($currentMaxNestingLevel < $recommendedMaxNestingLevel) {
                 $this->messageQueue->enqueue(new FlashMessage(
@@ -496,7 +528,7 @@ class Check implements CheckInterface
                         . ' raised further. A value of ' . $recommendedMaxNestingLevel . ' is recommended.' . LF
                         . 'xdebug.max_nesting_level=' . $recommendedMaxNestingLevel,
                     'PHP xdebug.max_nesting_level is low',
-                    FlashMessage::WARNING
+                    ContextualFeedbackSeverity::WARNING
                 ));
             } else {
                 $this->messageQueue->enqueue(new FlashMessage(
@@ -529,7 +561,7 @@ class Check implements CheckInterface
                     . ' to at least ' . $recommendedMaxInputVars . ':' . LF
                     . 'max_input_vars=' . $recommendedMaxInputVars,
                 'PHP max_input_vars too low',
-                FlashMessage::ERROR
+                ContextualFeedbackSeverity::ERROR
             ));
         } elseif ($currentMaxInputVars < $recommendedMaxInputVars) {
             $this->messageQueue->enqueue(new FlashMessage(
@@ -539,7 +571,7 @@ class Check implements CheckInterface
                     . ' to at least ' . $recommendedMaxInputVars . ':' . LF
                     . 'max_input_vars=' . $recommendedMaxInputVars,
                 'PHP max_input_vars very low',
-                FlashMessage::WARNING
+                ContextualFeedbackSeverity::WARNING
             ));
         } else {
             $this->messageQueue->enqueue(new FlashMessage(
@@ -571,9 +603,10 @@ class Check implements CheckInterface
                     . '* The PHP extension eaccelerator is known to break this if'
                     . ' it is compiled without --with-eaccelerator-doc-comment-inclusion flag.'
                     . ' This compile flag must be specified, otherwise TYPO3 CMS will not work.' . LF
-                    . 'For more information take a look in our documentation ' . Typo3Information::URL_OPCACHE . '.',
+                    . 'For more information take a look in our documentation ' .
+                    (new Typo3Information())->getDocsLink('t3coreapi:troubleshooting-php-troubleshooting-opcode') . '.',
                 'PHP Doc comment reflection broken',
-                FlashMessage::ERROR
+                ContextualFeedbackSeverity::ERROR
             ));
         } else {
             $this->messageQueue->enqueue(new FlashMessage(
@@ -589,7 +622,7 @@ class Check implements CheckInterface
     protected function checkWindowsApacheThreadStackSize()
     {
         if ($this->isWindowsOs()
-            && strpos($_SERVER['SERVER_SOFTWARE'], 'Apache') === 0
+            && str_starts_with($_SERVER['SERVER_SOFTWARE'], 'Apache')
         ) {
             $this->messageQueue->enqueue(new FlashMessage(
                 'This current value cannot be checked by the system, so please ignore this warning if it'
@@ -600,7 +633,7 @@ class Check implements CheckInterface
                     . ' to httpd.conf and restarting Apache afterwards:' . LF
                     . '<IfModule mpm_winnt_module>ThreadStackSize 8388608</IfModule>',
                 'Windows apache thread stack size',
-                FlashMessage::WARNING
+                ContextualFeedbackSeverity::WARNING
             ));
         } else {
             $this->messageQueue->enqueue(new FlashMessage(
@@ -612,10 +645,6 @@ class Check implements CheckInterface
 
     /**
      * Checks if a specific PHP extension is loaded.
-     *
-     * @param string $extension
-     * @param bool $required
-     * @param string $purpose
      */
     public function checkPhpExtension(string $extension, bool $required = true, string $purpose = '')
     {
@@ -625,7 +654,7 @@ class Check implements CheckInterface
                     . ' in your environment. Change your environment to provide this extension. '
                     . $purpose,
                 'PHP extension "' . $extension . '" not loaded',
-                $required ? FlashMessage::ERROR : FlashMessage::WARNING
+                $required ? ContextualFeedbackSeverity::ERROR : ContextualFeedbackSeverity::WARNING
             ));
         } else {
             $this->messageQueue->enqueue(new FlashMessage(
@@ -642,8 +671,7 @@ class Check implements CheckInterface
     {
         if (function_exists('imagecreatetruecolor')) {
             $imageResource = @imagecreatetruecolor(50, 100);
-            if (is_resource($imageResource)) {
-                imagedestroy($imageResource);
+            if ($this->checkImageResource($imageResource)) {
                 $this->messageQueue->enqueue(new FlashMessage(
                     '',
                     'PHP GD library true color works'
@@ -653,14 +681,14 @@ class Check implements CheckInterface
                     'GD is loaded, but calling imagecreatetruecolor() fails.'
                         . ' This must be fixed, TYPO3 CMS won\'t work well otherwise.',
                     'PHP GD library true color support broken',
-                    FlashMessage::ERROR
+                    ContextualFeedbackSeverity::ERROR
                 ));
             }
         } else {
             $this->messageQueue->enqueue(new FlashMessage(
                 'Gdlib is essential for TYPO3 CMS to work properly.',
                 'PHP GD library true color support missing',
-                FlashMessage::ERROR
+                ContextualFeedbackSeverity::ERROR
             ));
         }
     }
@@ -676,8 +704,7 @@ class Check implements CheckInterface
         ) {
             // Do not use data:// wrapper to be independent of allow_url_fopen
             $imageResource = @imagecreatefromgif(__DIR__ . '/../../Resources/Public/Images/TestInput/Test.gif');
-            if (is_resource($imageResource)) {
-                imagedestroy($imageResource);
+            if ($this->checkImageResource($imageResource)) {
                 $this->messageQueue->enqueue(new FlashMessage(
                     '',
                     'PHP GD library has gif support'
@@ -686,14 +713,14 @@ class Check implements CheckInterface
                 $this->messageQueue->enqueue(new FlashMessage(
                     'GD is loaded, but calling imagecreatefromgif() fails. This must be fixed, TYPO3 CMS won\'t work well otherwise.',
                     'PHP GD library gif support broken',
-                    FlashMessage::ERROR
+                    ContextualFeedbackSeverity::ERROR
                 ));
             }
         } else {
             $this->messageQueue->enqueue(new FlashMessage(
                 'GD must be compiled with gif support. This is essential for TYPO3 CMS to work properly.',
                 'PHP GD library gif support missing',
-                FlashMessage::ERROR
+                ContextualFeedbackSeverity::ERROR
             ));
         }
     }
@@ -715,7 +742,7 @@ class Check implements CheckInterface
             $this->messageQueue->enqueue(new FlashMessage(
                 'GD must be compiled with jpg support. This is essential for TYPO3 CMS to work properly.',
                 'PHP GD library jpg support missing',
-                FlashMessage::ERROR
+                ContextualFeedbackSeverity::ERROR
             ));
         }
     }
@@ -731,8 +758,7 @@ class Check implements CheckInterface
         ) {
             // Do not use data:// wrapper to be independent of allow_url_fopen
             $imageResource = @imagecreatefrompng(__DIR__ . '/../../Resources/Public/Images/TestInput/Test.png');
-            if (is_resource($imageResource)) {
-                imagedestroy($imageResource);
+            if ($this->checkImageResource($imageResource)) {
                 $this->messageQueue->enqueue(new FlashMessage(
                     '',
                     'PHP GD library has png support'
@@ -743,14 +769,14 @@ class Check implements CheckInterface
                         . ' Check your environment and fix it, png in GD lib is important'
                         . ' for TYPO3 CMS to work properly.',
                     'PHP GD library png support broken',
-                    FlashMessage::ERROR
+                    ContextualFeedbackSeverity::ERROR
                 ));
             }
         } else {
             $this->messageQueue->enqueue(new FlashMessage(
                 'GD must be compiled with png support. This is essential for TYPO3 CMS to work properly',
                 'PHP GD library png support missing',
-                FlashMessage::ERROR
+                ContextualFeedbackSeverity::ERROR
             ));
         }
     }
@@ -773,7 +799,7 @@ class Check implements CheckInterface
                     . ' to render fonts on images. This support is missing'
                     . ' in your environment. Install it.',
                 'PHP GD library freetype support missing',
-                FlashMessage::ERROR
+                ContextualFeedbackSeverity::ERROR
             ));
         }
     }
@@ -807,28 +833,6 @@ class Check implements CheckInterface
         return $windowsOs;
     }
 
-    /**
-     * Helper method to explode a string by delimiter and throw away empty values.
-     * Removes empty values from result array.
-     *
-     * @param string $delimiter Delimiter string to explode with
-     * @param string $string The string to explode
-     * @return array Exploded values
-     */
-    protected function trimExplode($delimiter, $string)
-    {
-        $explodedValues = explode($delimiter, $string);
-        $explodedValues = is_array($explodedValues) ? $explodedValues : [];
-        $resultWithPossibleEmptyValues = array_map('trim', $explodedValues);
-        $result = [];
-        foreach ($resultWithPossibleEmptyValues as $value) {
-            if ($value !== '') {
-                $result[] = $value;
-            }
-        }
-        return $result;
-    }
-
     /**
      * Helper method to get the bytes value from a measurement string like "100k".
      *
@@ -847,4 +851,9 @@ class Check implements CheckInterface
         }
         return (int)$bytes;
     }
+
+    private function checkImageResource($imageResource): bool
+    {
+        return $imageResource instanceof \GdImage;
+    }
 }
