@@ -15,8 +15,10 @@
 
 namespace TYPO3\CMS\Recycler\Domain\Model;
 
+use Symfony\Component\DependencyInjection\Attribute\Autoconfigure;
+use Symfony\Component\DependencyInjection\Attribute\Autowire;
+use TYPO3\CMS\Backend\Utility\BackendUtility;
 use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
-use TYPO3\CMS\Core\Cache\CacheManager;
 use TYPO3\CMS\Core\Cache\Frontend\FrontendInterface;
 use TYPO3\CMS\Core\Database\Connection;
 use TYPO3\CMS\Core\Database\ConnectionPool;
@@ -25,64 +27,42 @@ use TYPO3\CMS\Core\Database\Query\QueryBuilder;
 use TYPO3\CMS\Core\Database\Query\QueryHelper;
 use TYPO3\CMS\Core\Database\Query\Restriction\WorkspaceRestriction;
 use TYPO3\CMS\Core\DataHandling\DataHandler;
+use TYPO3\CMS\Core\Schema\Capability\LabelCapability;
+use TYPO3\CMS\Core\Schema\Capability\TcaSchemaCapability;
+use TYPO3\CMS\Core\Schema\TcaSchema;
+use TYPO3\CMS\Core\Schema\TcaSchemaFactory;
 use TYPO3\CMS\Core\Type\Bitmask\Permission;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\MathUtility;
-use TYPO3\CMS\Recycler\Utility\RecyclerUtility;
 
 /**
  * Model class for the 'recycler' extension.
  * @internal This class is a specific domain model implementation and is not part of the Public TYPO3 API.
  */
+#[Autoconfigure(public: true)]
 class DeletedRecords
 {
     /**
      * Array with all deleted rows
-     *
-     * @var array
      */
-    protected $deletedRows = [];
+    protected array $deletedRows = [];
 
     /**
      * String with the global limit
-     *
-     * @var string
      */
-    protected $limit = '';
+    protected string $limit = '';
 
     /**
-     * Array with all available FE tables
-     *
-     * @var array
+     * Array with all available tables
      */
-    protected $table = [];
+    protected array $table = [];
 
-    /**
-     * Object from helper class
-     *
-     * @var RecyclerUtility
-     */
-    protected $recyclerHelper;
+    public function __construct(
+        private readonly TcaSchemaFactory $tcaSchemaFactory,
+        #[Autowire(service: 'cache.runtime')]
+        protected readonly FrontendInterface $runtimeCache,
+    ) {}
 
-    /**
-     * Array with all label fields from different tables
-     *
-     * @var array
-     */
-    public $label;
-
-    /**
-     * Array with all title fields from different tables
-     *
-     * @var array
-     */
-    public $title;
-
-    /************************************************************
-     * GET DATA FUNCTIONS
-     *
-     *
-     ************************************************************/
     /**
      * Load all deleted rows from $table
      * If table is not set, it iterates the TCA tables
@@ -92,19 +72,19 @@ class DeletedRecords
      * @param int $depth How many levels recursive
      * @param string $limit MySQL LIMIT
      * @param string $filter Filter text
-     * @return DeletedRecords
      */
-    public function loadData($id, $table, $depth, $limit = '', $filter = '')
+    public function loadData($id, string $table, $depth, $limit = '', $filter = ''): self
     {
         // set the limit
         $this->limit = trim($limit);
         if ($table) {
-            if (in_array($table, RecyclerUtility::getModifyableTables(), true)) {
+            $schemata = $this->getRelevantSchemata();
+            if (array_key_exists($table, $schemata)) {
                 $this->table[] = $table;
-                $this->setData($id, $table, $depth, $filter);
+                $this->setData($id, $schemata[$table], $depth, $filter);
             }
         } else {
-            foreach (RecyclerUtility::getModifyableTables() as $tableKey) {
+            foreach ($this->getRelevantSchemata() as $tableKey => $schema) {
                 // only go into this table if the limit allows it
                 if ($this->limit !== '') {
                     $parts = GeneralUtility::intExplode(',', $this->limit, true);
@@ -114,7 +94,7 @@ class DeletedRecords
                     }
                 }
                 $this->table[] = $tableKey;
-                $this->setData($id, $tableKey, $depth, $filter);
+                $this->setData($id, $schema, $depth, $filter);
             }
         }
         return $this;
@@ -129,7 +109,7 @@ class DeletedRecords
      * @param string $filter Filter text
      * @return int
      */
-    public function getTotalCount($id, $table, $depth, $filter)
+    public function getTotalCount($id, $table, $depth, $filter): int
     {
         $deletedRecords = $this->loadData($id, $table, $depth, '', $filter)->getDeletedRows();
         $countTotal = 0;
@@ -143,38 +123,36 @@ class DeletedRecords
      * Set all deleted rows
      *
      * @param int $id UID from record
-     * @param string $table Tablename from record
      * @param int $depth How many levels recursive
      * @param string $filter Filter text
      */
-    protected function setData($id, $table, $depth, $filter)
+    protected function setData($id, TcaSchema $schema, $depth, $filter): void
     {
-        $deletedField = RecyclerUtility::getDeletedField($table);
+        $deletedField = $schema->getCapability(TcaSchemaCapability::SoftDelete)->getFieldName();
         if (!$deletedField) {
             return;
         }
 
         $id = (int)$id;
-        $tcaCtrl = $GLOBALS['TCA'][$table]['ctrl'];
         $firstResult = 0;
         $maxResults = 0;
 
         // get the limit
         if (!empty($this->limit)) {
             // count the number of deleted records for this pid
-            $queryBuilder = $this->getFilteredQueryBuilder($table, $id, $depth, $filter);
+            $queryBuilder = $this->getFilteredQueryBuilder($schema, $id, $depth, $filter);
 
             $deletedCount = (int)$queryBuilder
                 ->count('*')
-                ->from($table)
+                ->from($schema->getName())
                 ->andWhere(
                     $queryBuilder->expr()->neq(
                         $deletedField,
-                        $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT)
+                        $queryBuilder->createNamedParameter(0, Connection::PARAM_INT)
                     )
                 )
-                ->execute()
-                ->fetchColumn(0);
+                ->executeQuery()
+                ->fetchOne();
 
             // split the limit
             [$offset, $rowCount] = GeneralUtility::intExplode(',', $this->limit, true);
@@ -223,44 +201,42 @@ class DeletedRecords
         }
         // query for actual deleted records
         if ($allowQuery) {
-            $queryBuilder = $this->getFilteredQueryBuilder($table, $id, $depth, $filter);
+            $queryBuilder = $this->getFilteredQueryBuilder($schema, $id, $depth, $filter);
             if ($firstResult) {
                 $queryBuilder->setFirstResult($firstResult);
             }
             if ($maxResults) {
                 $queryBuilder->setMaxResults($maxResults);
             }
-            $recordsToCheck = $queryBuilder->select('*')
-                ->from($table)
+            $queryBuilder = $queryBuilder->select('*')
+                ->from($schema->getName())
                 ->andWhere(
                     $queryBuilder->expr()->eq(
                         $deletedField,
-                        $queryBuilder->createNamedParameter(1, \PDO::PARAM_INT)
+                        $queryBuilder->createNamedParameter(1, Connection::PARAM_INT)
                     )
-                )
-                ->orderBy('uid')
-                ->execute()
-                ->fetchAll();
+                );
 
-            if ($recordsToCheck !== false) {
-                $this->checkRecordAccess($table, $recordsToCheck);
+            if ($schema->hasCapability(TcaSchemaCapability::UpdatedAt)) {
+                $queryBuilder = $queryBuilder
+                    ->orderBy($schema->getCapability(TcaSchemaCapability::UpdatedAt)->getFieldName(), 'desc')
+                    ->addOrderBy('uid');
+            } else {
+                $queryBuilder = $queryBuilder->orderBy('uid');
+            }
+            $recordsToCheck = $queryBuilder->executeQuery()->fetchAllAssociative();
+            if ($recordsToCheck !== []) {
+                $this->checkRecordAccess($schema->getName(), $recordsToCheck);
             }
         }
-        $this->label[$table] = $tcaCtrl['label'];
-        $this->title[$table] = $tcaCtrl['title'];
     }
 
     /**
      * Helper method for setData() to create a QueryBuilder that filters the records by default.
-     *
-     * @param string $table
-     * @param int $pid
-     * @param int $depth
-     * @param string $filter
-     * @return \TYPO3\CMS\Core\Database\Query\QueryBuilder
      */
-    protected function getFilteredQueryBuilder(string $table, int $pid, int $depth, string $filter): QueryBuilder
+    protected function getFilteredQueryBuilder(TcaSchema $schema, int $pid, int $depth, string $filter): QueryBuilder
     {
+        $table = $schema->getName();
         $pidList = $this->getTreeList($pid, $depth);
         $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable($table);
         $queryBuilder->getRestrictions()->removeAll()
@@ -269,23 +245,24 @@ class DeletedRecords
         // create the filter WHERE-clause
         $filterConstraint = null;
         if (trim($filter) !== '') {
+            /** @var LabelCapability $labelCapability */
+            $labelCapability = $schema->getCapability(TcaSchemaCapability::Label);
             $filterConstraint = $queryBuilder->expr()->comparison(
-                $queryBuilder->castFieldToTextType($GLOBALS['TCA'][$table]['ctrl']['label']),
+                $queryBuilder->castFieldToTextType($labelCapability->getPrimaryFieldName()),
                 'LIKE',
                 $queryBuilder->createNamedParameter(
-                    '%' . $queryBuilder->escapeLikeWildcards($filter) . '%',
-                    \PDO::PARAM_STR
+                    '%' . $queryBuilder->escapeLikeWildcards($filter) . '%'
                 )
             );
             if (MathUtility::canBeInterpretedAsInteger($filter)) {
-                $filterConstraint = $queryBuilder->expr()->orX(
+                $filterConstraint = $queryBuilder->expr()->or(
                     $queryBuilder->expr()->eq(
                         'uid',
-                        $queryBuilder->createNamedParameter($filter, \PDO::PARAM_INT)
+                        $queryBuilder->createNamedParameter($filter, Connection::PARAM_INT)
                     ),
                     $queryBuilder->expr()->eq(
                         'pid',
-                        $queryBuilder->createNamedParameter($filter, \PDO::PARAM_INT)
+                        $queryBuilder->createNamedParameter($filter, Connection::PARAM_INT)
                     ),
                     $filterConstraint
                 );
@@ -301,9 +278,9 @@ class DeletedRecords
             );
         }
         $queryBuilder->where(
-            $queryBuilder->expr()->andX(
+            $queryBuilder->expr()->and(
                 $filterConstraint,
-                $queryBuilder->expr()->orX(...$pidConstraints)
+                $queryBuilder->expr()->or(...$pidConstraints)
             )
         );
 
@@ -316,28 +293,13 @@ class DeletedRecords
      * @param string $table Name of the table
      * @param array $rows Record row
      */
-    protected function checkRecordAccess($table, array $rows)
+    protected function checkRecordAccess(string $table, array $rows): void
     {
-        $deleteField = '';
-        if ($table === 'pages') {
-            // The "checkAccess" method validates access to the passed table/rows. When access to
-            // a page record gets validated it is necessary to disable the "delete" field temporarily
-            // for the recycler.
-            // Else it wouldn't be possible to perform the check as many methods of BackendUtility
-            // like "BEgetRootLine", etc. will only work on non-deleted records.
-            $deleteField = $GLOBALS['TCA'][$table]['ctrl']['delete'];
-            unset($GLOBALS['TCA'][$table]['ctrl']['delete']);
-        }
-
         foreach ($rows as $row) {
-            if (RecyclerUtility::checkAccess($table, $row)) {
+            if ($this->checkAccess($table, $row)) {
                 $this->setDeletedRows($table, $row);
             }
         }
-
-        if ($table === 'pages') {
-            $GLOBALS['TCA'][$table]['ctrl']['delete'] = $deleteField;
-        }
     }
 
     /************************************************************
@@ -347,18 +309,16 @@ class DeletedRecords
      * Delete element from any table
      *
      * @param array|null $recordsArray Representation of the records
-     * @return bool
      */
-    public function deleteData($recordsArray)
+    public function deleteData(?array $recordsArray): bool
     {
         if (is_array($recordsArray)) {
-            /** @var DataHandler $tce **/
+            /** @var DataHandler $tce */
             $tce = GeneralUtility::makeInstance(DataHandler::class);
             $tce->start([], []);
-            $tce->disableDeleteClause();
             foreach ($recordsArray as $record) {
                 [$table, $uid] = explode(':', $record);
-                $tce->deleteEl($table, (int)$uid, true, true);
+                $tce->deleteAction($table, (int)$uid, true, true);
             }
             return true;
         }
@@ -376,80 +336,77 @@ class DeletedRecords
      * @param bool $recursive Whether to recursively undelete
      * @return bool|int
      */
-    public function undeleteData($recordsArray, $recursive = false)
+    public function undeleteData(array $recordsArray, bool $recursive = false): bool|int
     {
         $result = false;
         $affectedRecords = 0;
         $depth = 999;
-        if (is_array($recordsArray)) {
-            $this->deletedRows = [];
-            $cmd = [];
-            foreach ($recordsArray as $record) {
-                [$table, $uid] = explode(':', $record);
-                $uid = (int)$uid;
-                // get all parent pages and cover them
-                $pid = RecyclerUtility::getPidOfUid($uid, $table);
-                if ($pid > 0) {
-                    $parentUidsToRecover = $this->getDeletedParentPages($pid);
-                    $count = count($parentUidsToRecover);
-                    for ($i = 0; $i < $count; ++$i) {
-                        $parentUid = $parentUidsToRecover[$i];
-                        $cmd['pages'][$parentUid]['undelete'] = 1;
-                        $affectedRecords++;
-                    }
-                    if (isset($cmd['pages'])) {
-                        // reverse the page list to recover it from top to bottom
-                        $cmd['pages'] = array_reverse($cmd['pages'], true);
-                    }
+        $this->deletedRows = [];
+        $cmd = [];
+        foreach ($recordsArray as $record) {
+            [$table, $uid] = explode(':', $record);
+            $uid = (int)$uid;
+            // get all parent pages and cover them
+            $pid = $this->getPidOfUid($uid, $table);
+            if ($pid > 0) {
+                $parentUidsToRecover = $this->getDeletedParentPages($pid);
+                $count = count($parentUidsToRecover);
+                for ($i = 0; $i < $count; ++$i) {
+                    $parentUid = $parentUidsToRecover[$i];
+                    $cmd['pages'][$parentUid]['undelete'] = 1;
+                    $affectedRecords++;
                 }
-                $cmd[$table][$uid]['undelete'] = 1;
-                $affectedRecords++;
-                if ($table === 'pages' && $recursive) {
-                    $this->loadData($uid, '', $depth, '');
-                    $childRecords = $this->getDeletedRows();
-                    if (!empty($childRecords)) {
-                        foreach ($childRecords as $childTable => $childRows) {
-                            foreach ($childRows as $childRow) {
-                                $cmd[$childTable][$childRow['uid']]['undelete'] = 1;
-                            }
+                if (isset($cmd['pages'])) {
+                    // reverse the page list to recover it from top to bottom
+                    $cmd['pages'] = array_reverse($cmd['pages'], true);
+                }
+            }
+            $cmd[$table][$uid]['undelete'] = 1;
+            $affectedRecords++;
+            if ($table === 'pages' && $recursive) {
+                $this->loadData($uid, '', $depth, '');
+                $childRecords = $this->getDeletedRows();
+                if (!empty($childRecords)) {
+                    foreach ($childRecords as $childTable => $childRows) {
+                        foreach ($childRows as $childRow) {
+                            $cmd[$childTable][$childRow['uid']]['undelete'] = 1;
                         }
                     }
                 }
             }
-            if ($cmd) {
-                $tce = GeneralUtility::makeInstance(DataHandler::class);
-                $tce->start([], $cmd);
-                $tce->process_cmdmap();
-                $result = $affectedRecords;
-            }
+        }
+        if ($cmd) {
+            $tce = GeneralUtility::makeInstance(DataHandler::class);
+            $tce->start([], $cmd);
+            $tce->process_cmdmap();
+            $result = $affectedRecords;
         }
         return $result;
     }
 
     /**
      * Returns deleted parent pages
-     *
-     * @param int $uid
-     * @param array $pages
-     * @return array
      */
-    protected function getDeletedParentPages($uid, &$pages = [])
+    protected function getDeletedParentPages(int $uid, array &$pages = []): array
     {
         $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('pages');
         $queryBuilder->getRestrictions()->removeAll()
             ->add(GeneralUtility::makeInstance(WorkspaceRestriction::class, $this->getBackendUser()->workspace));
+
+        $deletedField = $this->tcaSchemaFactory->get('pages')->getCapability(TcaSchemaCapability::SoftDelete)->getFieldName();
+
         $record = $queryBuilder
             ->select('uid', 'pid')
             ->from('pages')
             ->where(
-                $queryBuilder->expr()->eq('uid', $queryBuilder->createNamedParameter($uid, \PDO::PARAM_INT)),
-                $queryBuilder->expr()->eq($GLOBALS['TCA']['pages']['ctrl']['delete'], 1)
+                $queryBuilder->expr()->eq('uid', $queryBuilder->createNamedParameter($uid, Connection::PARAM_INT)),
+                $queryBuilder->expr()->eq($deletedField, 1)
             )
-            ->execute()
-            ->fetch();
+            ->executeQuery()
+            ->fetchAssociative();
         if ($record) {
             $pages[] = $record['uid'];
-            if ((int)$record['pid'] !== 0) {
+            if ((int)$record['pid'] > 0) {
                 $this->getDeletedParentPages($record['pid'], $pages);
             }
         }
@@ -457,80 +414,36 @@ class DeletedRecords
         return $pages;
     }
 
-    /************************************************************
-     * SETTER FUNCTIONS
-     ************************************************************/
     /**
-     * Set deleted rows
-     *
-     * @param string $table Tablename
      * @param array $row Deleted record row
      */
-    public function setDeletedRows($table, array $row)
+    protected function setDeletedRows(string $table, array $row): void
     {
         $this->deletedRows[$table][] = $row;
     }
 
-    /************************************************************
-     * GETTER FUNCTIONS
-     ************************************************************/
-    /**
-     * Get deleted Rows
-     *
-     * @return array Array with all deleted rows from TCA
-     */
-    public function getDeletedRows()
+    public function getDeletedRows(): array
     {
         return $this->deletedRows;
     }
 
-    /**
-     * Get table
-     *
-     * @return array Array with table from TCA
-     */
-    public function getTable()
+    protected function getTreeList(int $id, int $depth): array
     {
-        return $this->table;
-    }
-
-    /**
-     * Get tree list
-     *
-     * @param int $id
-     * @param int $depth
-     * @param int $begin
-     * @return array
-     */
-    protected function getTreeList(int $id, int $depth, int $begin = 0): array
-    {
-        $cache = $this->getCache();
-        $identifier = md5($id . '_' . $depth . '_' . $begin);
-        $pageTree = $cache->get($identifier);
+        $identifier = md5($id . '_' . $depth);
+        $pageTree = $this->runtimeCache->get($identifier);
         if ($pageTree === false) {
-            $pageTree = $this->resolveTree($id, $depth, $begin, $this->getBackendUser()->getPagePermsClause(Permission::PAGE_SHOW));
-            $cache->set($identifier, $pageTree);
+            $pageTree = $this->resolveTree($id, $depth, $this->getBackendUser()->getPagePermsClause(Permission::PAGE_SHOW));
+            $pageTree = array_merge([$id], $pageTree);
+            $this->runtimeCache->set($identifier, $pageTree);
         }
 
         return $pageTree;
     }
 
-    /**
-     * @param int $id
-     * @param int $depth
-     * @param int $begin
-     * @param string $permsClause
-     * @return array
-     */
-    protected function resolveTree(int $id, int $depth, int $begin = 0, string $permsClause = ''): array
+    protected function resolveTree(int $id, int $depth, string $permsClause): array
     {
-        $depth = (int)$depth;
-        $begin = (int)$begin;
-        $id = abs((int)$id);
+        $id = abs($id);
         $theList = [];
-        if ($begin === 0) {
-            $theList[] = $id;
-        }
         if ($depth > 0) {
             $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('pages');
             $queryBuilder->getRestrictions()->removeAll()
@@ -538,16 +451,14 @@ class DeletedRecords
             $statement = $queryBuilder->select('uid')
                 ->from('pages')
                 ->where(
-                    $queryBuilder->expr()->eq('pid', $queryBuilder->createNamedParameter($id, \PDO::PARAM_INT)),
+                    $queryBuilder->expr()->eq('pid', $queryBuilder->createNamedParameter($id, Connection::PARAM_INT)),
                     QueryHelper::stripLogicalOperatorPrefix($permsClause)
                 )
-                ->execute();
-            while ($row = $statement->fetch()) {
-                if ($begin <= 0) {
-                    $theList[] = $row['uid'];
-                }
+                ->executeQuery();
+            while ($row = $statement->fetchAssociative()) {
+                $theList[] = $row['uid'];
                 if ($depth > 1) {
-                    $theList = array_merge($theList, $this->resolveTree($row['uid'], $depth - 1, $begin - 1, $permsClause));
+                    $theList = array_merge($theList, $this->resolveTree($row['uid'], $depth - 1, $permsClause));
                 }
             }
         }
@@ -555,22 +466,92 @@ class DeletedRecords
     }
 
     /**
-     * Gets an instance of the memory cache.
-     *
-     * @return FrontendInterface
+     * Get pid of uid
      */
-    protected function getCache(): FrontendInterface
+    protected function getPidOfUid(int $uid, string $table): int
     {
-        return GeneralUtility::makeInstance(CacheManager::class)->getCache('runtime');
+        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable($table);
+        $queryBuilder->getRestrictions()->removeAll();
+
+        $pid = $queryBuilder
+            ->select('pid')
+            ->from($table)
+            ->where($queryBuilder->expr()->eq('uid', $queryBuilder->createNamedParameter($uid, Connection::PARAM_INT)))
+            ->executeQuery()
+            ->fetchOne();
+
+        return (int)$pid;
+    }
+    /**
+     * Checks the page access rights (Code for access check mostly taken from FormEngine)
+     * as well as the table access rights of the user.
+     *
+     * @param string $table The table to check access for
+     * @param array $row Record array
+     * @return bool Returns TRUE is the user has access, or FALSE if not
+     */
+    protected function checkAccess(string $table, array $row): bool
+    {
+        $backendUser = $this->getBackendUser();
+
+        if ($backendUser->isAdmin()) {
+            return true;
+        }
+
+        if (!$backendUser->check('tables_modify', $table)) {
+            return false;
+        }
+
+        // Checking if the user has permissions? (Only working as a precaution, because the final permission check is always down in TCE. But it's good to notify the user on beforehand...)
+        // First, resetting flags.
+        $hasAccess = false;
+        $calcPRec = $row;
+        BackendUtility::workspaceOL($table, $calcPRec, $backendUser->workspace);
+        if (is_array($calcPRec)) {
+            if ($table === 'pages') {
+                $calculatedPermissions = new Permission($backendUser->calcPerms($calcPRec, false));
+                $hasAccess = $calculatedPermissions->editPagePermissionIsGranted();
+            } else {
+                $rec = BackendUtility::getRecord('pages', $calcPRec['pid'], '*', '', false);
+                $calculatedPermissions = new Permission($backendUser->calcPerms(BackendUtility::getRecord('pages', $calcPRec['pid'], '*', '', false), false));
+                // Fetching pid-record first.
+                $hasAccess = $calculatedPermissions->editContentPermissionIsGranted();
+            }
+            // Check internals regarding access:
+            if ($hasAccess) {
+                $hasAccess = $backendUser->recordEditAccessInternals($table, $calcPRec);
+            }
+        }
+        return $hasAccess;
     }
 
-    /**
-     * Returns the BackendUser
-     *
-     * @return BackendUserAuthentication
-     */
     protected function getBackendUser(): BackendUserAuthentication
     {
         return $GLOBALS['BE_USER'];
     }
+
+    /**
+     * Returns the modifiable tables of the current user, which have a SoftDelete field.
+     *
+     * @return TcaSchema[]
+     */
+    protected function getRelevantSchemata(): array
+    {
+        $schemata = [];
+        $tables = explode(',', $this->getBackendUser()->groupData['tables_modify']);
+        foreach ($this->tcaSchemaFactory->all() as $name => $schema) {
+            if (!$schema->hasCapability(TcaSchemaCapability::SoftDelete)) {
+                continue;
+            }
+            if ($this->getBackendUser()->isAdmin()) {
+                $schemata[$name] = $schema;
+                continue;
+            }
+            if (in_array($name, $tables, true)) {
+                $schemata[$name] = $schema;
+            }
+        }
+        return $schemata;
+    }
+
 }
