@@ -17,102 +17,88 @@ declare(strict_types=1);
 
 namespace TYPO3\CMS\Redirects\Hooks;
 
+use Symfony\Component\DependencyInjection\Attribute\Autoconfigure;
 use TYPO3\CMS\Backend\Utility\BackendUtility;
 use TYPO3\CMS\Core\DataHandling\DataHandler;
 use TYPO3\CMS\Core\Utility\MathUtility;
+use TYPO3\CMS\Redirects\RedirectUpdate\SlugRedirectChangeItem;
+use TYPO3\CMS\Redirects\RedirectUpdate\SlugRedirectChangeItemFactory;
 use TYPO3\CMS\Redirects\Service\SlugService;
 
 /**
  * @internal This class is a specific TYPO3 hook implementation and is not part of the Public TYPO3 API.
  */
+#[Autoconfigure(public: true)]
 class DataHandlerSlugUpdateHook
 {
-    /**
-     * @var SlugService
-     */
-    protected $slugService;
-
     /**
      * Persisted slug values per record UID
-     * e.g. `[13 => 'slug-a', 14 => 'slug-x/example']`
+     * e.g. `[13 => SlugRedirectChangeItem( $original = ['slug' => 'slug-a'] ), 14 => SlugRedirectChangeItem( $original = ['slug' => 'slug-x/example'] )`
      *
-     * @var string[]
+     * @var array<int, SlugRedirectChangeItem>
      */
-    protected $persistedSlugValues;
+    protected $persistedChangedItems;
 
-    /**
-     * @param SlugService $slugService
-     */
-    public function __construct(SlugService $slugService)
-    {
-        $this->slugService = $slugService;
-    }
+    public function __construct(
+        protected SlugService $slugService,
+        protected SlugRedirectChangeItemFactory $slugRedirectChangeItemFactory,
+    ) {}
 
     /**
      * Collects slugs of persisted records before having been updated.
      *
-     * @param array $incomingFieldArray
-     * @param string $table
      * @param string|int $id (id could be string, for this reason no type hint)
-     * @param DataHandler $dataHandler
      */
     public function processDatamap_preProcessFieldArray(array $incomingFieldArray, string $table, $id, DataHandler $dataHandler): void
     {
-        if (
-            $table !== 'pages'
+        if ($table !== 'pages'
             || empty($incomingFieldArray['slug'])
             || $this->isNestedHookInvocation($dataHandler)
             || !MathUtility::canBeInterpretedAsInteger($id)
-            || !$dataHandler->checkRecordUpdateAccess($table, $id, $incomingFieldArray)
+            || !$dataHandler->hasPermissionToUpdate('pages', BackendUtility::getRecord('pages', (int)$id) ?? [])
         ) {
             return;
         }
-
-        $record = BackendUtility::getRecordWSOL($table, (int)$id, 'slug');
-        $this->persistedSlugValues[(int)$id] = $record['slug'];
+        $changeItem = $this->slugRedirectChangeItemFactory->create((int)$id);
+        if ($changeItem === null) {
+            return;
+        }
+        $this->persistedChangedItems[(int)$id] = $changeItem;
     }
 
     /**
      * Acts on potential slug changes.
      *
-     * Hook `processDatamap_postProcessFieldArray` is executed after `DataHandler::fillInFields` which
-     * ensure access to pages.slug field and applies possible evaluations (`eval => 'trim,...`).
-     *
-     * @param string $status
-     * @param string $table
-     * @param string|int $id
-     * @param array $fieldArray
-     * @param DataHandler $dataHandler
+     * Hook `processDatamap_afterDatabaseOperations` is a record has been persisted and after `DataHandler::fillInFields`
+     * which ensure access to `pages.slug` field and applies possible evaluations (`eval => 'trim,...`).
      */
-    public function processDatamap_postProcessFieldArray(string $status, string $table, $id, array $fieldArray, DataHandler $dataHandler): void
+    public function processDatamap_afterDatabaseOperations(string $status, string $table, $id, array $fieldArray, DataHandler $dataHandler): void
     {
-        $persistedSlugValue = $this->persistedSlugValues[(int)$id] ?? null;
+        $persistedChangedItem = $this->persistedChangedItems[(int)$id] ?? null;
 
         if (
-            $table !== 'pages'
+            $persistedChangedItem === null
+            || $table !== 'pages'
             || $status !== 'update'
             || empty($fieldArray['slug'])
-            || $persistedSlugValue === null
-            || $persistedSlugValue === $fieldArray['slug']
+            || $persistedChangedItem->getOriginal()['slug'] === $fieldArray['slug']
             || $this->isNestedHookInvocation($dataHandler)
         ) {
             return;
         }
-
-        $this->slugService->rebuildSlugsForSlugChange($id, $persistedSlugValue, $fieldArray['slug'], $dataHandler->getCorrelationId());
+        // We merge the fieldArray dataset into with the original record to spare a database query here.
+        $persistedChangedItem = $persistedChangedItem->withChanged(array_merge($persistedChangedItem->getOriginal(), $fieldArray));
+        $this->slugService->rebuildSlugsForSlugChange($id, $persistedChangedItem, $dataHandler->getCorrelationId());
     }
 
     /**
      * Determines whether our identifier is part of correlation id aspects.
      * In that case it would be a nested call which has to be ignored.
-     *
-     * @param DataHandler $dataHandler
-     * @return bool
      */
     protected function isNestedHookInvocation(DataHandler $dataHandler): bool
     {
         $correlationId = $dataHandler->getCorrelationId();
-        $correlationIdAspects = $correlationId ? $correlationId->getAspects() ?? [] : [];
+        $correlationIdAspects = $correlationId ? $correlationId->getAspects() : [];
         return in_array(SlugService::CORRELATION_ID_IDENTIFIER, $correlationIdAspects, true);
     }
 }
