@@ -1,5 +1,4 @@
 <?php
-namespace TYPO3\CMS\Reports\Report\Status;
 
 /*
  * This file is part of the TYPO3 CMS project.
@@ -13,8 +12,13 @@ namespace TYPO3\CMS\Reports\Report\Status;
  *
  * The TYPO3 project - inspiring people to share!
  */
+
+namespace TYPO3\CMS\Reports\Report\Status;
+
 use TYPO3\CMS\Core\Localization\LanguageService;
+use TYPO3\CMS\Core\Type\ContextualFeedbackSeverity;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
+use TYPO3\CMS\Reports\Status;
 use TYPO3\CMS\Reports\Status as ReportStatus;
 use TYPO3\CMS\Reports\StatusProviderInterface;
 
@@ -26,9 +30,9 @@ class Typo3Status implements StatusProviderInterface
     /**
      * Returns the status for this report
      *
-     * @return array List of statuses
+     * @return Status[] List of statuses
      */
-    public function getStatus()
+    public function getStatus(): array
     {
         $statuses = [
             'registeredXclass' => $this->getRegisteredXclassStatus(),
@@ -36,6 +40,11 @@ class Typo3Status implements StatusProviderInterface
         return $statuses;
     }
 
+    public function getLabel(): string
+    {
+        return 'typo3';
+    }
+
     /**
      * List any Xclasses registered in the system
      *
@@ -44,8 +53,8 @@ class Typo3Status implements StatusProviderInterface
     protected function getRegisteredXclassStatus()
     {
         $message = '';
-        $value = $this->getLanguageService()->getLL('status_none');
-        $severity = ReportStatus::OK;
+        $value = $this->getLanguageService()->sL('LLL:EXT:reports/Resources/Private/Language/locallang_reports.xlf:status_none');
+        $severity = ContextualFeedbackSeverity::OK;
 
         $xclassFoundArray = [];
         if (array_key_exists('Objects', $GLOBALS['TYPO3_CONF_VARS']['SYS'])) {
@@ -56,34 +65,31 @@ class Typo3Status implements StatusProviderInterface
             }
         }
         if (!empty($xclassFoundArray)) {
-            $value = $this->getLanguageService()->getLL('status_xclassUsageFound');
-            $message = $this->getLanguageService()->getLL('status_xclassUsageFound_message') . '<br />';
+            $value = $this->getLanguageService()->sL('LLL:EXT:reports/Resources/Private/Language/locallang_reports.xlf:status_xclassUsageFound');
+            $message = $this->getLanguageService()->sL('LLL:EXT:reports/Resources/Private/Language/locallang_reports.xlf:status_xclassUsageFound_message') . '<br />';
             $message .= '<ol>';
             foreach ($xclassFoundArray as $originalClass => $xClassName) {
                 $messageDetail = sprintf(
-                    $this->getLanguageService()->getLL('status_xclassUsageFound_message_detail'),
-                    '<code>' . htmlspecialchars($originalClass) . '</code>',
-                    '<code>' . htmlspecialchars($xClassName) . '</code>'
+                    (string)($this->getLanguageService()->sL('LLL:EXT:reports/Resources/Private/Language/locallang_reports.xlf:status_xclassUsageFound_message_detail')),
+                    '<code>' . htmlspecialchars((string)$originalClass) . '</code>',
+                    '<code>' . htmlspecialchars((string)$xClassName) . '</code>'
                 );
                 $message .= '<li>' . $messageDetail . '</li>';
             }
             $message .= '</ol>';
-            $severity = ReportStatus::NOTICE;
+            $severity = ContextualFeedbackSeverity::NOTICE;
         }
 
         return GeneralUtility::makeInstance(
             ReportStatus::class,
-            $this->getLanguageService()->getLL('status_xclassUsage'),
+            $this->getLanguageService()->sL('LLL:EXT:reports/Resources/Private/Language/locallang_reports.xlf:status_xclassUsage'),
             $value,
             $message,
             $severity
         );
     }
 
-    /**
-     * @return LanguageService
-     */
-    protected function getLanguageService()
+    protected function getLanguageService(): LanguageService
     {
         return $GLOBALS['LANG'];
     }
